@@ -1,4 +1,3 @@
-
 /*
  * bltBitmap.c --
  *
@@ -18,26 +17,25 @@
  * suitability of this software for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
+ * Copyright 1993-1997 Bell Labs Innovations for Lucent Technologies.
  *
- * Copyright 1993-1996 by AT&T Bell Laboratories.
- * Permission to use, copy, modify, and distribute this software
- * and its documentation for any purpose and without fee is hereby
- * granted, provided that the above copyright notice appear in all
- * copies and that both that the copyright notice and warranty
- * disclaimer appear in supporting documentation, and that the
- * names of AT&T Bell Laboratories any of their entities not be used
- * in advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation for any purpose and without fee is hereby granted, provided
+ * that the above copyright notice appear in all copies and that both that the
+ * copyright notice and warranty disclaimer appear in supporting documentation,
+ * and that the names of Lucent Technologies any of their entities not be used
+ * in advertising or publicity pertaining to distribution of the software
+ * without specific, written prior permission.
  *
- * AT&T disclaims all warranties with regard to this software, including
- * all implied warranties of merchantability and fitness.  In no event
- * shall AT&T be liable for any special, indirect or consequential
- * damages or any damages whatsoever resulting from loss of use, data
- * or profits, whether in an action of contract, negligence or other
- * tortuous action, arising out of or in connection with the use or
- * performance of this software.
+ * Lucent Technologies disclaims all warranties with regard to this software,
+ * including all implied warranties of merchantability and fitness.  In no event
+ * shall Lucent Technologies be liable for any special, indirect or
+ * consequential damages or any damages whatsoever resulting from loss of use,
+ * data or profits, whether in an action of contract, negligence or other
+ * tortuous action, arising out of or in connection with the use or performance
+ * of this software.
  *
- * "bitmap" command created by George Howlett.
+ * The "bitmap" command created by George Howlett.
  */
 
 #include "bltInt.h"
@@ -53,7 +51,7 @@ extern Tk_CustomOption bltPadOption;
 typedef struct {
     double rotate;		/* Rotation of text string */
     double scale;		/* Scaling factor */
-    XFontStruct *fontPtr;	/* Font pointer */
+    Tk_Font font;		/* Font pointer */
     Tk_Justify justify;		/* Justify text */
     Pad padX, padY;		/* Padding around the text */
 } BitmapInfo;
@@ -75,7 +73,7 @@ typedef struct {
 static Tk_ConfigSpec composeConfigSpecs[] =
 {
     {TK_CONFIG_FONT, "-font", (char *)NULL, (char *)NULL,
-	DEF_BITMAP_FONT, Tk_Offset(BitmapInfo, fontPtr), 0},
+	DEF_BITMAP_FONT, Tk_Offset(BitmapInfo, font), 0},
     {TK_CONFIG_JUSTIFY, "-justify", (char *)NULL, (char *)NULL,
 	DEF_BITMAP_JUSTIFY, Tk_Offset(BitmapInfo, justify),
 	TK_CONFIG_DONT_SET_DEFAULT},
@@ -285,8 +283,7 @@ GetHexValue(interp, string, valuePtr)
  *	Converts a bitmap into an data array.
  *
  * Results:
- *	Returns the number of bytes in an data array representing
- * 	the bitmap.  Returns -1 if the array could not be allocated.
+ *	Returns the number of bytes in an data array representing the bitmap.  
  *
  * Side Effects:
  *	Memory is allocated for the data array. Caller must free
@@ -295,8 +292,7 @@ GetHexValue(interp, string, valuePtr)
  * -----------------------------------------------------------------------
  */
 static int
-BitmapToData(interp, tkwin, bitmap, width, height, dataPtrPtr)
-    Tcl_Interp *interp;		/* Interpreter to report results to */
+BitmapToData(tkwin, bitmap, width, height, dataPtrPtr)
     Tk_Window tkwin;		/* Main window of interpreter */
     Pixmap bitmap;		/* Bitmap to be queried */
     int width, height;		/* Dimensions of the bitmap */
@@ -319,10 +315,7 @@ BitmapToData(interp, tkwin, bitmap, width, height, dataPtrPtr)
     bytes_per_line = (width + 7) / 8;
     arraySize = height * bytes_per_line;
     dataPtr = (unsigned char *)malloc(sizeof(unsigned char) * arraySize);
-    if (dataPtr == NULL) {
-	interp->result = "can't allocate bitmap data array";
-	return -1;
-    }
+    assert(dataPtr);
     count = 0;
     for (y = 0; y < height; y++) {
 	value = 0, bitMask = 1;
@@ -344,7 +337,7 @@ BitmapToData(interp, tkwin, bitmap, width, height, dataPtrPtr)
     XDestroyImage(imagePtr);
 
     *dataPtrPtr = dataPtr;
-    return (count);
+    return count;
 }
 
 /*
@@ -399,8 +392,7 @@ AsciiToData(interp, elemList, width, height, dataPtrPtr)
     } else if (numValues == (arraySize / 2)) {
 	format = V10;
     } else {
-	Tcl_AppendResult(interp, "bitmap has wrong # of data values",
-	    (char *)NULL);
+	Tcl_AppendResult(interp, "bitmap has wrong # of data values", (char *)NULL);
 	goto error;
     }
     padding = 0;
@@ -527,8 +519,8 @@ RotateData(interp, srcPtr, theta, destPtr)
 		rotX += srcX;
 		rotY += srcY;
 
-		sx = BLT_RND(rotX);
-		sy = BLT_RND(rotY);
+		sx = ROUND(rotX);
+		sy = ROUND(rotY);
 
 		/*
 		 * Verify the coordinates, since the destination image can be
@@ -606,8 +598,8 @@ ScaleData(interp, srcPtr, scale, destPtr)
 	    scaleX = (dx / scale);
 	    scaleY = (dy / scale);
 
-	    sx = BLT_RND(scaleX);
-	    sy = BLT_RND(scaleY);
+	    sx = ROUND(scaleX);
+	    sy = ROUND(scaleY);
 
 	    /*
 	     * Verify the coordinates, since the destination image can be
@@ -648,11 +640,11 @@ ScaleData(interp, srcPtr, scale, destPtr)
  *
  * -----------------------------------------------------------------------
  */
-static int
-ShowBitmap(interp, tkwin, bitmap)
-    Tcl_Interp *interp;		/* Interpreter to report results to */
+static void
+ShowBitmap(tkwin, bitmap, dsPtr)
     Tk_Window tkwin;		/* Main window of interpreter */
     Pixmap bitmap;		/* Bitmap to be queried */
+    Tcl_DString *dsPtr;		/* Dynamic string to output results to */
 {
     unsigned char *dataPtr;
     char *separator;
@@ -663,24 +655,20 @@ ShowBitmap(interp, tkwin, bitmap)
 
     /* Get the dimensions of the bitmap */
     Tk_SizeOfBitmap(Tk_Display(tkwin), bitmap, &width, &height);
-    arraySize = BitmapToData(interp, tkwin, bitmap, width, height, &dataPtr);
-    if (arraySize < 0) {
-	return TCL_ERROR;
-    }
+    arraySize = BitmapToData(tkwin, bitmap, width, height, &dataPtr);
 #define BYTES_PER_OUTPUT_LINE 24
     for (i = 0; i < arraySize; i++) {
 	separator = (i % BYTES_PER_OUTPUT_LINE) ? " " : "\n    ";
-	sprintf(string, "%02x", dataPtr[i]);
-	Tcl_AppendResult(interp, separator, string, 0);
+	sprintf(string, "%s%02x", separator, dataPtr[i]);
+	Tcl_DStringAppend(dsPtr, string, -1);
     }
     free((char *)dataPtr);
-    return TCL_OK;
 }
 
 /*
  *--------------------------------------------------------------
  *
- * ComposeOper --
+ * ComposeOp --
  *
  *	Converts the text string into an internal bitmap.
  *
@@ -700,13 +688,13 @@ ShowBitmap(interp, tkwin, bitmap)
  *--------------------------------------------------------------
  */
 static int
-ComposeOper(tkwin, interp, argc, argv)
+ComposeOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* Number of arguments */
     char **argv;		/* Argument list */
 {
-    int width, height;		/* dimensions of bitmap */
+    int bmWidth, bmHeight;	/* Dimensions of bitmap */
     Pixmap bitmap;		/* Text bitmap */
     unsigned char *dataPtr;	/* Data array derived from text bitmap */
     int arraySize;
@@ -714,45 +702,46 @@ ComposeOper(tkwin, interp, argc, argv)
     int result;
     double theta;
     TextAttributes textAttr;
+    int dummy;
 
     /* Initialize info and process flags */
-    info.fontPtr = NULL;	/* Will be initialized by Tk_ConfigureWidget */
-    info.rotate = 0.0;		/* No rotation by default */
-    info.scale = 1.0;
     info.justify = TK_JUSTIFY_CENTER;
+    info.rotate = 0.0;		/* No rotation or scaling by default */
+    info.scale = 1.0;
     info.padLeft = info.padRight = 0;
     info.padTop = info.padBottom = 0;
+    info.font = (Tk_Font)NULL;	/* Will be initialized by Tk_ConfigureWidget */
     if (Tk_ConfigureWidget(interp, tkwin, composeConfigSpecs,
 	    argc - 4, argv + 4, (char *)&info, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
-    theta = BLT_FMOD(info.rotate, (double)360.0);
+    theta = FMOD(info.rotate, (double)360.0);
     if (theta < 0.0) {
 	theta += 360.0;
     }
-    textAttr.fontPtr = info.fontPtr;
+    textAttr.font = info.font;
     textAttr.theta = theta;
     textAttr.justify = info.justify;
     textAttr.padX = info.padX;
     textAttr.padY = info.padY;
+    textAttr.leader = 0;
+    textAttr.anchor = TK_ANCHOR_CENTER;
 
-    bitmap = Blt_CreateTextBitmap(tkwin, argv[3], &textAttr, &width, &height);
+    bitmap = Blt_CreateTextBitmap(tkwin, argv[3], &textAttr, &dummy, &dummy,
+		  &bmWidth, &bmHeight);
 
     /* Free the font structure, since we don't need it anymore */
     Tk_FreeOptions(composeConfigSpecs, (char *)&info, Tk_Display(tkwin), 0);
 
     /* Convert bitmap back to a data array */
-    arraySize = BitmapToData(interp, tkwin, bitmap, width, height, &dataPtr);
+    arraySize = BitmapToData(tkwin, bitmap, bmWidth, bmHeight, &dataPtr);
     Tk_FreePixmap(Tk_Display(tkwin), bitmap);
-    if (arraySize < 0) {
-	return TCL_ERROR;
-    }
     if (info.scale != 1.0) {
 	BitmapData srcData, destData;
 
 	srcData.dataArr = dataPtr;
-	srcData.width = width;
-	srcData.height = height;
+	srcData.width = bmWidth;
+	srcData.height = bmHeight;
 	srcData.arraySize = arraySize;
 
 	result = ScaleData(interp, &srcData, info.scale, &destData);
@@ -761,12 +750,12 @@ ComposeOper(tkwin, interp, argc, argv)
 	    return TCL_ERROR;
 	}
 	dataPtr = destData.dataArr;
-	width = destData.width;
-	height = destData.height;
+	bmWidth = destData.width;
+	bmHeight = destData.height;
     }
     /* Create the bitmap again, this time using Tk's bitmap facilities */
     result = Tk_DefineBitmap(interp, Tk_GetUid(argv[2]), (char *)dataPtr,
-	width, height);
+			     bmWidth, bmHeight);
     if (result != TCL_OK) {
 	free((char *)dataPtr);
     }
@@ -776,7 +765,7 @@ ComposeOper(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * DefineOper --
+ * DefineOp --
  *
  *	Converts the dataList into an internal bitmap.
  *
@@ -791,7 +780,7 @@ ComposeOper(tkwin, interp, argc, argv)
  */
 /* ARGSUSED */
 static int
-DefineOper(tkwin, interp, argc, argv)
+DefineOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* Number of arguments */
@@ -861,7 +850,7 @@ DefineOper(tkwin, interp, argc, argv)
     free((char *)elemArr);
     elemArr = NULL;
 
-    theta = BLT_FMOD(info.rotate, 360.0);
+    theta = FMOD(info.rotate, 360.0);
     if (theta < 0.0) {
 	theta += 360.0;
     }
@@ -923,7 +912,7 @@ DefineOper(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * ExistOper --
+ * ExistOp --
  *
  *	Indicates if the named bitmap exists.
  *
@@ -931,7 +920,7 @@ DefineOper(tkwin, interp, argc, argv)
  */
 /*ARGSUSED*/
 static int
-ExistsOper(tkwin, interp, argc, argv)
+ExistsOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* not used */
@@ -953,7 +942,7 @@ ExistsOper(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * HeightOper --
+ * HeightOp --
  *
  *	Returns the height of the named bitmap.
  *
@@ -961,7 +950,7 @@ ExistsOper(tkwin, interp, argc, argv)
  */
 /*ARGSUSED*/
 static int
-HeightOper(tkwin, interp, argc, argv)
+HeightOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* not used */
@@ -983,7 +972,7 @@ HeightOper(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * WidthOper --
+ * WidthOp --
  *
  *	Returns the width of the named bitmap.
  *
@@ -991,7 +980,7 @@ HeightOper(tkwin, interp, argc, argv)
  */
 /*ARGSUSED*/
 static int
-WidthOper(tkwin, interp, argc, argv)
+WidthOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* not used */
@@ -1013,7 +1002,7 @@ WidthOper(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * SourceOper --
+ * SourceOp --
  *
  *	Returns the data array (excluding width and height)
  *	of the named bitmap.
@@ -1022,29 +1011,30 @@ WidthOper(tkwin, interp, argc, argv)
  */
 /*ARGSUSED*/
 static int
-SourceOper(tkwin, interp, argc, argv)
+SourceOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* not used */
     char **argv;		/* Argument list */
 {
     Pixmap bitmap;
+    Tcl_DString dStr;
 
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
     }
-    if (ShowBitmap(interp, tkwin, bitmap) != TCL_OK) {
-	return TCL_ERROR;
-    }
+    Tcl_DStringInit(&dStr);
+    ShowBitmap(tkwin, bitmap, &dStr);
     Tk_FreeBitmap(Tk_Display(tkwin), bitmap);
+    Tcl_DStringResult(interp, &dStr);
     return TCL_OK;
 }
 
 /*
  *--------------------------------------------------------------
  *
- * DataOper --
+ * DataOp --
  *
  *	Returns the data array, including width and height,
  *	of the named bitmap.
@@ -1053,7 +1043,7 @@ SourceOper(tkwin, interp, argc, argv)
  */
 /*ARGSUSED*/
 static int
-DataOper(tkwin, interp, argc, argv)
+DataOp(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* not used */
@@ -1062,19 +1052,20 @@ DataOper(tkwin, interp, argc, argv)
     Pixmap bitmap;
     char string[200];
     int width, height;
+    Tcl_DString dStr;
 
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
     }
     Tk_SizeOfBitmap(Tk_Display(tkwin), bitmap, &width, &height);
-    sprintf(string, "%d %d", width, height);
-    Tcl_AppendResult(interp, "  {", string, "} {", (char *)NULL);
-    if (ShowBitmap(interp, tkwin, bitmap) != TCL_OK) {
-	return TCL_ERROR;
-    }
-    Tcl_AppendResult(interp, "\n  }", (char *)NULL);
+    Tcl_DStringInit(&dStr);
+    sprintf(string, "  {%d %d} {", width, height);
+    Tcl_DStringAppend(&dStr, string, -1);
+    ShowBitmap(tkwin, bitmap, &dStr);
     Tk_FreeBitmap(Tk_Display(tkwin), bitmap);
+    Tcl_DStringAppend(&dStr, "\n  }", -1);
+    Tcl_DStringResult(interp, &dStr);
     return TCL_OK;
 }
 
@@ -1093,19 +1084,19 @@ DataOper(tkwin, interp, argc, argv)
  *
  *--------------------------------------------------------------
  */
-static Blt_OperSpec operSpecs[] =
+static Blt_OpSpec bitmapOps[] =
 {
-    {"compose", 1, (Blt_OperProc) ComposeOper, 4, 0,
+    {"compose", 1, (Blt_Operation) ComposeOp, 4, 0,
 	"bitmapName text ?flags?",},
-    {"data", 2, (Blt_OperProc) DataOper, 3, 3, "bitmapName",},
-    {"define", 2, (Blt_OperProc) DefineOper, 4, 0,
+    {"data", 2, (Blt_Operation) DataOp, 3, 3, "bitmapName",},
+    {"define", 2, (Blt_Operation) DefineOp, 4, 0,
 	"bitmapName data ?flags?",},
-    {"exists", 1, (Blt_OperProc) ExistsOper, 3, 3, "bitmapName",},
-    {"height", 1, (Blt_OperProc) HeightOper, 3, 3, "bitmapName",},
-    {"source", 1, (Blt_OperProc) SourceOper, 3, 3, "bitmapName",},
-    {"width", 1, (Blt_OperProc) WidthOper, 3, 3, "bitmapName",},
+    {"exists", 1, (Blt_Operation) ExistsOp, 3, 3, "bitmapName",},
+    {"height", 1, (Blt_Operation) HeightOp, 3, 3, "bitmapName",},
+    {"source", 1, (Blt_Operation) SourceOp, 3, 3, "bitmapName",},
+    {"width", 1, (Blt_Operation) WidthOp, 3, 3, "bitmapName",},
 };
-static int numSpecs = sizeof(operSpecs) / sizeof(Blt_OperSpec);
+static int numBitmapOps = sizeof(bitmapOps) / sizeof(Blt_OpSpec);
 
 /*
  *--------------------------------------------------------------
@@ -1133,10 +1124,10 @@ BitmapCmd(clientData, interp, argc, argv)
     char **argv;
 {
     Tk_Window tkwin = (Tk_Window)clientData;
-    Blt_OperProc proc;
+    Blt_Operation proc;
     int result;
 
-    proc = Blt_LookupOperation(interp, numSpecs, operSpecs, BLT_OPER_ARG1,
+    proc = Blt_LookupOperation(interp, numBitmapOps, bitmapOps, BLT_OPER_ARG1,
 	argc, argv);
     if (proc == NULL) {
 	return TCL_ERROR;
@@ -1165,11 +1156,11 @@ int
 Blt_BitmapInit(interp)
     Tcl_Interp *interp;
 {
-    static Blt_CmdSpec cmdSpec = { "bitmap", BITMAP_VERSION, BitmapCmd, };
+    static Blt_CmdSpec cmdSpec = { "bitmap", BitmapCmd, };
 
     /* Define the BLT logo bitmaps */
 
-    if (Blt_InitCmd(interp, &cmdSpec) != TCL_OK) {
+    if (Blt_InitCmd(interp, "blt", &cmdSpec) != TCL_OK) {
 	return TCL_ERROR;
     }
     Tk_DefineBitmap(interp, Tk_GetUid("BLT"), (char *)blt_bits,
