@@ -557,12 +557,12 @@ QuadEval(splX, splY, x, y, m, len, splLen, epsilon)
 				 * evaluated. */
     double splY[];		/* (out) To be filled with the images
 				 * of the points in splX. */
-    double x[];			/* Abscissas of the data * points to
+    double x[];			/* Abscissas of the data points to
 				 * be interpolated. X must be increasing. */
     double y[];			/* Ordinates of the data points to be
 				 * interpolated. */
     double m[];			/* Slope of the spline at each point
-				 * of interpolation.  */
+				 * of interpolation. */
     int len;			/* Number of data points (dimension of
 				 * X and y). */
     int splLen;			/* Number of points of evaluation
@@ -822,7 +822,7 @@ NaturalSpline(x, y, len, splX, splY, splLen, work)
     for (i = 0; i < end; i++) {
 	h[i] = x[i + 1] - x[i];
 	if (h[i] < 0.0) {
-	    return TCL_ERROR;
+	    return -1;
 	}
     }
 
@@ -872,33 +872,59 @@ NaturalSpline(x, y, len, splX, splY, splLen, work)
 	    splY[j] = a[loc] + v * (b[loc] + v * (c[loc] + v * d[loc]));
 	}
     }
-    return TCL_OK;
+    return 0;
 }
 
-/*ARGSUSED*/
-static int
-NaturalOper(tkwin, interp, xPtr, yPtr, splXPtr, splYPtr, argc, argv)
-    Tk_Window tkwin;		/* Main window of the interpreter. Used to
-				 * process options using Tk_ConfigureWidget */
-    Tcl_Interp *interp;
-    Blt_Vector *xPtr, *yPtr, *splXPtr, *splYPtr;
-    int argc;			/* Not used */
-    char **argv;		/* Not used */
+int
+Blt_NaturalSpline(x, y, length, splX, splY, splLen)
+    double x[], y[];
+    int length;
+    double splX[], splY[];
+    int splLen;
 {
     double *work;
     int result;
 
-    /* Allocate space for vectors used in calculation */
-
-    work = (double *)malloc(sizeof(double) * xPtr->numValues * 9);
-    if (work == NULL) {
-	interp->result = "can't allocate working storage for spline";
-	return TCL_ERROR;
-    }
-    result = NaturalSpline(xPtr->valueArr, yPtr->valueArr, xPtr->numValues,
-	splXPtr->valueArr, splYPtr->valueArr, splXPtr->numValues, work);
+    work = (double *)malloc(sizeof(double) * length * 9);
+    assert(work);
+    result = NaturalSpline(x, y, length, splX, splY, splLen, work);
     free((char *)work);
-    if (result != TCL_OK) {
+    return (result);
+}
+
+int
+Blt_QuadraticSpline(x, y, length, splX, splY, splLen, epsilon)
+    double x[], y[];
+    int length;
+    double splX[], splY[];
+    int splLen;
+    double epsilon;
+{
+    double *work;
+    int result;
+
+    /* allocate space for vectors used in calculation */
+    work = (double *)malloc(length * sizeof(double));
+    assert(work);
+    result = QuadraticSpline(x, y, length, splX, splY, splLen, work, epsilon);
+    free((char *)work);
+    return (result);
+}
+
+
+
+/*ARGSUSED*/
+static int
+NaturalOp(tkwin, interp, x, y, splX, splY, argc, argv)
+    Tk_Window tkwin;		/* Main window of the interpreter. Used to
+				 * process options using Tk_ConfigureWidget */
+    Tcl_Interp *interp;
+    Blt_Vector *x, *y, *splX, *splY;
+    int argc;			/* Not used */
+    char **argv;		/* Not used */
+{
+    if (Blt_NaturalSpline(Blt_VecData(x), Blt_VecData(y), Blt_VecLength(x),
+	Blt_VecData(splX), Blt_VecData(splY), Blt_VecLength(splX)) != 0) {
 	Tcl_AppendResult(interp, "x vector \"", argv[2],
 	    "\" must be sorted in ascending order", (char *)NULL);
 	return TCL_ERROR;
@@ -906,6 +932,7 @@ NaturalOper(tkwin, interp, xPtr, yPtr, splXPtr, splYPtr, argc, argv)
     return TCL_OK;
 }
 
+
 typedef struct {
     double epsilon;		/* Error setting for calculating the spline */
 } QuadInfo;
@@ -923,16 +950,15 @@ static Tk_ConfigSpec quadConfigSpecs[] =
 
 /* ARGSUSED */
 static int
-QuadraticOper(tkwin, interp, xPtr, yPtr, splXPtr, splYPtr, argc, argv)
+QuadraticOp(tkwin, interp, x, y, splX, splY, argc, argv)
     Tk_Window tkwin;		/* Main window of the interpreter. Used to
 				 * process options using Tk_ConfigureWidget */
     Tcl_Interp *interp;
-    Blt_Vector *xPtr, *yPtr, *splXPtr, *splYPtr;
+    Blt_Vector *x, *y, *splX, *splY;
     int argc;			/* Not used */
     char **argv;
 {
     double epsilon;
-    double *work;
     int result;
 
     epsilon = 0.0;		/* TBA: adjust error via command-line option */
@@ -949,16 +975,9 @@ QuadraticOper(tkwin, interp, xPtr, yPtr, splXPtr, splYPtr, argc, argv)
 	}
 	epsilon = info.epsilon;
     }
-    /* allocate space for vectors used in calculation */
-    work = (double *)malloc(xPtr->numValues * sizeof(double));
-    if (work == NULL) {
-	interp->result = "can't allocate work array";
-	return TCL_ERROR;
-    }
-    result = QuadraticSpline(xPtr->valueArr, yPtr->valueArr, xPtr->numValues,
-	splXPtr->valueArr, splYPtr->valueArr, splXPtr->numValues, work,
-	epsilon);
-    free((char *)work);
+    result = Blt_QuadraticSpline(Blt_VecData(x), Blt_VecData(y), 
+	Blt_VecLength(x), Blt_VecData(splX), Blt_VecData(splY), 
+	Blt_VecLength(splX), epsilon);
     if (result != 0) {
 	sprintf(interp->result, "error #%d in \"%s\"", result, argv[2]);
 	return TCL_ERROR;
@@ -966,14 +985,14 @@ QuadraticOper(tkwin, interp, xPtr, yPtr, splXPtr, splYPtr, argc, argv)
     return TCL_OK;
 }
 
-static Blt_OperSpec operSpecs[] =
+static Blt_OpSpec operSpecs[] =
 {
-    {"natural", 1, (Blt_OperProc) NaturalOper, 6, 0,
+    {"natural", 1, (Blt_Operation) NaturalOp, 6, 0,
 	"x y splx sply ?option value?...",},
-    {"quadratic", 1, (Blt_OperProc) QuadraticOper, 6, 0,
+    {"quadratic", 1, (Blt_Operation) QuadraticOp, 6, 0,
 	"x y splx sply ?option value?...",},
 };
-static int numSpecs = sizeof(operSpecs) / sizeof(Blt_OperSpec);
+static int numSpecs = sizeof(operSpecs) / sizeof(Blt_OpSpec);
 
 static int
 SplineCmd(clientData, interp, argc, argv)
@@ -983,8 +1002,8 @@ SplineCmd(clientData, interp, argc, argv)
     char **argv;
 {
     Tk_Window tkwin = (Tk_Window)clientData;
-    Blt_OperProc proc;
-    Blt_Vector x, y, splX, splY;
+    Blt_Operation proc;
+    Blt_Vector *x, *y, *splX, *splY;
     register int i;
     int result;
 
@@ -998,19 +1017,19 @@ SplineCmd(clientData, interp, argc, argv)
 	(Blt_GetVector(interp, argv[4], &splX) != TCL_OK)) {
 	return TCL_ERROR;
     }
-    if (x.numValues < 3) {
+    if (Blt_VecLength(x) < 3) {
 	Tcl_AppendResult(interp, "length of vector \"", argv[2], "\" is < 3",
 	    (char *)NULL);
 	return TCL_ERROR;
     }
-    for (i = 1; i < x.numValues; i++) {
-	if (x.valueArr[i] <= x.valueArr[i - 1]) {
+    for (i = 1; i < Blt_VecLength(x); i++) {
+	if (Blt_VecData(x)[i] <= Blt_VecData(x)[i - 1]) {
 	    Tcl_AppendResult(interp, "x vector \"", argv[2],
 		"\" must be monotonically increasing", (char *)NULL);
 	    return TCL_ERROR;
 	}
     }
-    if (x.numValues != y.numValues) {
+    if (Blt_VecLength(x) != Blt_VecLength(y)) {
 	Tcl_AppendResult(interp, "vectors \"", argv[2], "\" and \"", argv[3],
 	    " have different lengths", (char *)NULL);
 	return TCL_ERROR;
@@ -1021,25 +1040,20 @@ SplineCmd(clientData, interp, argc, argv)
 	 * doesn't exist, create one the same size as the vector
 	 * containing the abscissas.
 	 */
-	if (Blt_CreateVector(interp, argv[5], splX.numValues,
+	if (Blt_CreateVector(interp, argv[5], Blt_VecLength(splX),
 		&splY) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-    } else if (splX.numValues != splY.numValues) {
+    } else if (Blt_VecLength(splX) != Blt_VecLength(splY)) {
 	/*
 	 * The x and y vectors differ in size. Make the number of ordinates
 	 * the same as the number of abscissas.
 	 */
-
-	if (Blt_ResizeVector(interp, argv[5], splX.numValues) != TCL_OK) {
-	    return TCL_ERROR;
-	}
-	/* Re-get the vector, since its size has now changed */
-	if (Blt_GetVector(interp, argv[5], &splY) != TCL_OK) {
+	if (Blt_ResizeVector(splY, Blt_VecLength(splX)) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     }
-    result = (*proc) (tkwin, interp, &x, &y, &splX, &splY, argc, argv);
+    result = (*proc) (tkwin, interp, x, y, splX, splY, argc, argv);
     if (result != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -1049,7 +1063,8 @@ SplineCmd(clientData, interp, argc, argv)
      * memory is still the same).  The vector does not need to be
      * reallocated (TCL_STATIC is ignored).
      */
-    if (Blt_ResetVector(interp, argv[5], &splY, TCL_STATIC) != TCL_OK) {
+    if (Blt_ResetVector(splY, Blt_VecData(splY), Blt_VecLength(splY), 
+		Blt_VecSize(splY), TCL_STATIC) != TCL_OK) {
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -1059,7 +1074,7 @@ int
 Blt_SplineInit(interp)
     Tcl_Interp *interp;
 {
-    static Blt_CmdSpec cmdSpec = { "spline", SPLINE_VERSION, SplineCmd, };
+    static Blt_CmdSpec cmdSpec = { "spline", SplineCmd, };
 
-    return (Blt_InitCmd(interp, &cmdSpec));
+    return (Blt_InitCmd(interp, "blt", &cmdSpec));
 }
