@@ -1,76 +1,84 @@
+
 /*
  * bltUtil.c --
  *
  *	This module implements utility procedures for the BLT
  *	toolkit.
  *
- * Copyright 1993-1996 by AT&T Bell Laboratories.
- * Permission to use, copy, modify, and distribute this software
- * and its documentation for any purpose and without fee is hereby
- * granted, provided that the above copyright notice appear in all
- * copies and that both that the copyright notice and warranty
- * disclaimer appear in supporting documentation, and that the
- * names of AT&T Bell Laboratories any of their entities not be used
- * in advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
+ * Copyright 1991-1997 Bell Labs Innovations for Lucent Technologies.
  *
- * AT&T disclaims all warranties with regard to this software, including
- * all implied warranties of merchantability and fitness.  In no event
- * shall AT&T be liable for any special, indirect or consequential
- * damages or any damages whatsoever resulting from loss of use, data
- * or profits, whether in an action of contract, negligence or other
- * tortuous action, arising out of or in connection with the use or
- * performance of this software.
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation for any purpose and without fee is hereby granted, provided
+ * that the above copyright notice appear in all copies and that both that the
+ * copyright notice and warranty disclaimer appear in supporting documentation,
+ * and that the names of Lucent Technologies any of their entities not be used
+ * in advertising or publicity pertaining to distribution of the software
+ * without specific, written prior permission.
  *
+ * Lucent Technologies disclaims all warranties with regard to this software,
+ * including all implied warranties of merchantability and fitness.  In no event
+ * shall Lucent Technologies be liable for any special, indirect or
+ * consequential damages or any damages whatsoever resulting from loss of use,
+ * data or profits, whether in an action of contract, negligence or other
+ * tortuous action, arising out of or in connection with the use or performance
+ * of this software.  
  */
 
 #include "bltInt.h"
+#include <ctype.h>
 #include <X11/Xutil.h>
+#include <X11/Xproto.h>
+#if defined(__STDC__)
+#include <stdarg.h>
+#else
+#include <varargs.h>
+#endif
 
-static int FillParseProc _ANSI_ARGS_((ClientData clientData,
+static int FillParse _ANSI_ARGS_((ClientData clientData,
 	Tcl_Interp *interp, Tk_Window tkwin, char *value, char *widgRec,
 	int flags));
-static char *FillPrintProc _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+static char *FillPrint _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
 	Tcl_FreeProc **));
 
 Tk_CustomOption bltFillOption =
 {
-    FillParseProc, FillPrintProc, (ClientData)0
+    FillParse, FillPrint, (ClientData)0
 };
 
-static int PadParseProc _ANSI_ARGS_((ClientData clientData,
+static int PadParse _ANSI_ARGS_((ClientData clientData,
 	Tcl_Interp *interp, Tk_Window tkwin, char *value, char *widgRec,
 	int offset));
-static char *PadPrintProc _ANSI_ARGS_((ClientData clientData, Tk_Window tkwin,
+static char *PadPrint _ANSI_ARGS_((ClientData clientData, Tk_Window tkwin,
 	char *widgRec, int offset, Tcl_FreeProc **freeProcPtr));
 
 Tk_CustomOption bltPadOption =
 {
-    PadParseProc, PadPrintProc, (ClientData)0
+    PadParse, PadPrint, (ClientData)0
 };
 
-static int LengthParseProc _ANSI_ARGS_((ClientData clientData,
+static int LengthParse _ANSI_ARGS_((ClientData clientData,
 	Tcl_Interp *interp, Tk_Window tkwin, char *value, char *widgRec,
 	int flags));
-static char *LengthPrintProc _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+static char *LengthPrint _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
 	Tcl_FreeProc **));
 
 Tk_CustomOption bltLengthOption =
 {
-    LengthParseProc, LengthPrintProc, (ClientData)0
+    LengthParse, LengthPrint, (ClientData)0
 };
 
 
-static int DashesParseProc _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
+static int DashesParse _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
 	char *, char *, int));
-static char *DashesPrintProc _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+static char *DashesPrint _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
 	Tcl_FreeProc **));
 
 Tk_CustomOption bltDashesOption =
 {
-    DashesParseProc, DashesPrintProc, (ClientData)0
+    DashesParse, DashesPrint, (ClientData)0
 };
 
+
 /*
  *----------------------------------------------------------------------
  *
@@ -94,29 +102,29 @@ Blt_NameOfFill(fill)
     case FILL_BOTH:
 	return "both";
     default:
-	return "unknown fill value";
+	return "unknown value";
     }
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * FillParseProc --
+ * FillParse --
  *
  *	Converts the fill style string into its numeric representation.
  *
  *	Valid style strings are:
  *
- *	  "none"   Don't expand the window to fill the cubicle.
- * 	  "x"	   Expand only the window's width.
- *	  "y"	   Expand only the window's height.
- *	  "both"   Expand both the window's height and width.
+ *	  "none"   Use neither plane.
+ * 	  "x"	   X-coordinate plane.
+ *	  "y"	   Y-coordinate plane.
+ *	  "both"   Use both coordinate planes.
  *
  *----------------------------------------------------------------------
  */
 /*ARGSUSED*/
 static int
-FillParseProc(clientData, interp, tkwin, value, widgRec, offset)
+FillParse(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* not used */
@@ -139,7 +147,7 @@ FillParseProc(clientData, interp, tkwin, value, widgRec, offset)
     } else if ((c == 'b') && (strncmp(value, "both", length) == 0)) {
 	*fillPtr = FILL_BOTH;
     } else {
-	Tcl_AppendResult(interp, "bad fill argument \"", value,
+	Tcl_AppendResult(interp, "bad argument \"", value,
 	    "\": should be \"none\", \"x\", \"y\", or \"both\"", (char *)NULL);
 	return TCL_ERROR;
     }
@@ -149,7 +157,7 @@ FillParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * FillPrintProc --
+ * FillPrint --
  *
  *	Returns the fill style string based upon the fill flags.
  *
@@ -160,11 +168,11 @@ FillParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 /*ARGSUSED*/
 static char *
-FillPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+FillPrint(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
-    char *widgRec;		/* Row/column structure record */
-    int offset;			/* Offset of fill in Partition record */
+    char *widgRec;		/* Widget structure record */
+    int offset;			/* Offset of fill in widget record */
     Tcl_FreeProc **freeProcPtr;	/* not used */
 {
     Fill fill = *(Fill *)(widgRec + offset);
@@ -172,6 +180,69 @@ FillPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     return (Blt_NameOfFill(fill));
 }
 
+#ifdef notdef
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_FlagParse --
+ *
+ *	Converts the fill style string into its numeric representation.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+int
+Blt_FlagParse(clientData, interp, tkwin, value, widgRec, offset)
+    ClientData clientData;	/* Bit mask to be tested in status word */
+    Tcl_Interp *interp;		/* Interpreter to send results back to */
+    Tk_Window tkwin;		/* not used */
+    char *value;		/* Fill style string */
+    char *widgRec;		/* Cubicle structure record */
+    int offset;			/* Offset of style in record */
+{
+    unsigned int mask = (unsigned int)clientData;	/* Bit to be tested */
+    int *flagPtr = (int *)(widgRec + offset);
+    int bool;
+
+    if (Tcl_GetBoolean(interp, value, &bool) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    if (bool) {
+	*flagPtr |= mask;
+    } else {
+	*flagPtr &= ~mask;
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_FlagPrint --
+ *
+ *	Returns the fill style string based upon the fill flags.
+ *
+ * Results:
+ *	The fill style string is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+char *
+Blt_FlagPrint(clientData, tkwin, widgRec, offset, freeProcPtr)
+    ClientData clientData;	/* Bit mask to be test in status word */
+    Tk_Window tkwin;		/* not used */
+    char *widgRec;		/* Widget structure record */
+    int offset;			/* Offset of fill in widget record */
+    Tcl_FreeProc **freeProcPtr;	/* Unused */
+{
+    unsigned int mask = (unsigned int)clientData;	/* Bit to be tested */
+    unsigned int bool = *(unsigned int *)(widgRec + offset);
+
+    return (bool & mask) ? "1" : "0";
+}
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
@@ -180,7 +251,7 @@ FillPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  *	Like Tk_GetPixels, but doesn't allow negative pixel values.
  *
  * Results:
- *	The fill style string is returned.
+ *	A standard Tcl result.
  *
  *----------------------------------------------------------------------
  */
@@ -198,8 +269,8 @@ Blt_GetLength(interp, tkwin, string, valuePtr)
 	return TCL_ERROR;
     }
     if (length < 0) {
-	Tcl_AppendResult(interp, "screen distance \"", string,
-	    "\" must be non-negative value", (char *)NULL);
+	Tcl_AppendResult(interp, "can't have negative screen distance \"", string,
+	    "\"", (char *)NULL);
 	return TCL_ERROR;
     }
     *valuePtr = length;
@@ -209,7 +280,7 @@ Blt_GetLength(interp, tkwin, string, valuePtr)
 /*
  *----------------------------------------------------------------------
  *
- * LengthParseProc --
+ * LengthParse --
  *
  *	Like TK_CONFIG_PIXELS, but adds an extra check for negative
  *	values.
@@ -218,7 +289,7 @@ Blt_GetLength(interp, tkwin, string, valuePtr)
  */
 /*ARGSUSED*/
 static int
-LengthParseProc(clientData, interp, tkwin, value, widgRec, offset)
+LengthParse(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window */
@@ -234,7 +305,7 @@ LengthParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * LengthPrintProc --
+ * LengthPrint --
  *
  *	Returns the string representing the positive pixel size.
  *
@@ -245,30 +316,28 @@ LengthParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 /*ARGSUSED*/
 static char *
-LengthPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+LengthPrint(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
-    char *widgRec;		/* Row/column structure record */
-    int offset;			/* Offset of fill in Partition record */
+    char *widgRec;		/* Widget structure record */
+    int offset;			/* Offset in widget record */
     Tcl_FreeProc **freeProcPtr;	/* not used */
 {
     int length = *(int *)(widgRec + offset);
-    char *resultPtr;
+    char *result;
     char string[200];
 
     sprintf(string, "%d", length);
-    resultPtr = strdup(string);
-    if (resultPtr == NULL) {
-	return (strerror(errno));
-    }
+    result = strdup(string);
+    assert(result);
     *freeProcPtr = (Tcl_FreeProc *)free;
-    return (resultPtr);
+    return (result);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * PadParseProc --
+ * PadParse --
  *
  *	Convert a string into two pad values.  The string may be in one of
  *	the following forms:
@@ -290,7 +359,7 @@ LengthPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 /*ARGSUSED*/
 static int
-PadParseProc(clientData, interp, tkwin, value, widgRec, offset)
+PadParse(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window */
@@ -331,7 +400,7 @@ PadParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * PadPrintProc --
+ * PadPrint --
  *
  *	Converts the two pad values into a Tcl list.  Each pad has two
  *	pixel values.  For vertical pads, they represent the top and bottom
@@ -345,7 +414,7 @@ PadParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 /*ARGSUSED*/
 static char *
-PadPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+PadPrint(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Structure record */
@@ -359,7 +428,7 @@ PadPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     sprintf(string, "%d %d", padPtr->side1, padPtr->side2);
     result = strdup(string);
     if (result == NULL) {
-	return (strerror(errno));
+	return "out of memory";
     }
     *freeProcPtr = (Tcl_FreeProc *)free;
     return (result);
@@ -410,7 +479,7 @@ GetDashes(interp, string, dashesPtr)
 	goto badDashList;
     }
     for (i = 0; i < numValues; i++) {
-	if (Tcl_ExprLong(interp, strArr[i], &value) != TCL_OK) {
+	if (Blt_ExprLong(interp, strArr[i], &value) != TCL_OK) {
 	    goto badDashList;
 	}
 	if ((value < 1) || (value > 255)) {
@@ -422,9 +491,9 @@ GetDashes(interp, string, dashesPtr)
 		"\" is out of range", (char *)NULL);
 	    goto badDashList;
 	}
-	dashesPtr->valueList[i] = (unsigned char)value;
+	dashesPtr->valueArr[i] = (unsigned char)value;
     }
-    dashesPtr->valueList[i] = '\0';
+    dashesPtr->valueArr[i] = '\0';
     dashesPtr->numValues = i;
     if (numValues > 0) {
 	free((char *)strArr);
@@ -441,7 +510,7 @@ GetDashes(interp, string, dashesPtr)
 /*
  *----------------------------------------------------------------------
  *
- * DashesParseProc --
+ * DashesParse --
  *
  *	Convert the list of dash values into a dashes array.
  *
@@ -455,7 +524,7 @@ GetDashes(interp, string, dashesPtr)
  */
 /*ARGSUSED*/
 static int
-DashesParseProc(clientData, interp, tkwin, value, widgRec, offset)
+DashesParse(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* not used */
@@ -471,7 +540,7 @@ DashesParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * PositionPrintProc --
+ * DashesPrint --
  *
  *	Convert the dashes array into a list of values.
  *
@@ -482,7 +551,7 @@ DashesParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 /*ARGSUSED*/
 static char *
-DashesPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+DashesPrint(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget record */
@@ -490,7 +559,7 @@ DashesPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;	/* Memory deallocation scheme to use */
 {
     Dashes *dashesPtr = (Dashes *)(widgRec + offset);
-    Tcl_DString dashList;
+    Tcl_DString dStr;
     register int i;
     char *result;
     char string[200];
@@ -498,25 +567,43 @@ DashesPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     if (dashesPtr->numValues == 0) {
 	return "";
     }
-    Tcl_DStringInit(&dashList);
+    Tcl_DStringInit(&dStr);
     for (i = 0; i < dashesPtr->numValues; i++) {
-	sprintf(string, "%d", (int)dashesPtr->valueList[i]);
-	Tcl_DStringAppendElement(&dashList, string);
+	sprintf(string, "%d", (int)dashesPtr->valueArr[i]);
+	Tcl_DStringAppendElement(&dStr, string);
     }
+    result = Tcl_DStringValue(&dStr);
+    if (result == dStr.staticSpace) {
+	result = strdup(result);
+    } 
     *freeProcPtr = (Tcl_FreeProc *)free;
-    result = strdup(Tcl_DStringValue(&dashList));
-    Tcl_DStringFree(&dashList);
     return (result);
 }
 
+#ifndef NDEBUG
+    
+void
+Blt_Assert(testExpr, fileName, lineNumber)
+    char *testExpr;
+    char *fileName;
+    int lineNumber;
+{
+    fprintf(stderr, "line %d of %s: Assert \"%s\" failed",
+	    lineNumber, fileName, testExpr);
+    abort();
+}
+
+#endif
+
+
 /*
  *----------------------------------------------------------------------
  *
  * Blt_ConfigModified --
  *
- *      Given the specs and option argument names (terminated by a NULL),
- *	search  a Tk_ConfigSpec structure for any matching specs which
- *      has been reset.
+ *      Given the configuration specifications and one or more option 
+ *	patterns (terminated by a NULL), indicate if any of the matching
+ *	configuration options has been reset.
  *
  * Results:
  *      Returns 1 if one of the options has changed, 0 otherwise.
@@ -524,90 +611,35 @@ DashesPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  *----------------------------------------------------------------------
  */
 int
-Blt_ConfigModified(specs, opt0, opt1, opt2, opt3, opt4, opt5, opt6, opt7,
-    opt8, opt9)
-    Tk_ConfigSpec specs[];
-    char *opt0, *opt1, *opt2, *opt3, *opt4, *opt5, *opt6, *opt7, *opt8;
-    char *opt9;
+Blt_ConfigModified TCL_VARARGS_DEF(Tk_ConfigSpec *, arg1)
 {
+    va_list argList;
+    Tk_ConfigSpec *specs;
     register Tk_ConfigSpec *specPtr;
-    char *argv[11];
-    register char **argPtr;
+    register char *option;
 
-    argv[0] = opt0;
-    argv[1] = opt1;
-    argv[2] = opt2;
-    argv[3] = opt3;
-    argv[4] = opt4;
-    argv[5] = opt5;
-    argv[6] = opt6;
-    argv[7] = opt7;
-    argv[8] = opt8;
-    argv[9] = opt9;
-    argv[10] = NULL;
-    for (specPtr = specs; specPtr->type != TK_CONFIG_END; specPtr++) {
-	for (argPtr = argv; *argPtr != NULL; argPtr++) {
-	    if ((Tcl_StringMatch(specPtr->argvName, *argPtr)) &&
+    specs = TCL_VARARGS_START(Tk_ConfigSpec *, arg1, argList);
+    for (;;) {
+	option = va_arg(argList, char *);
+	if (option == NULL) {
+	    break;
+	}
+	for (specPtr = specs; specPtr->type != TK_CONFIG_END; specPtr++) {
+	    if ((Tcl_StringMatch(specPtr->argvName, option)) &&
 		(specPtr->specFlags & TK_CONFIG_OPTION_SPECIFIED)) {
+		va_end(argList);
 		return 1;
 	    }
 	}
     }
+    va_end(argList);
     return 0;
 }
 
-#if (TK_MAJOR_VERSION == 3)
-
 /*
  *----------------------------------------------------------------------
  *
- * Blt_ConfigureValue --
- *
- *      Roughly duplicates Tk 4.0's Tk_ConfigureValue function.
- *
- * Results:
- *      Returns a standard Tcl result.  *interp->result* will contain
- *	the configuration option value.
- *
- *----------------------------------------------------------------------
- */
-int
-Blt_ConfigureValue(interp, tkwin, specs, widRec, option, flags)
-    Tcl_Interp *interp;
-    Tk_Window tkwin;
-    Tk_ConfigSpec specs[];
-    char *widRec;
-    char *option;
-    int flags;
-{
-    int numValues;
-    char **valueArr;
-    char *resultPtr;
-    int status;
-
-    if (Tk_ConfigureInfo(interp, tkwin, specs, widRec,
-	    option, flags) != TCL_OK) {
-	return TCL_ERROR;
-    }
-    resultPtr = strdup(interp->result);
-    status = Tcl_SplitList(interp, resultPtr, &numValues, &valueArr);
-    free(resultPtr);
-
-    if (status != TCL_OK) {
-	return TCL_ERROR;
-    }
-    resultPtr = strdup(valueArr[4]);
-    Tcl_SetResult(interp, resultPtr, TCL_DYNAMIC);
-    free((char *)valueArr);
-    return TCL_OK;
-}
-
-#endif /* TK_MAJOR_VERSION == 3 */
-
-/*
- *----------------------------------------------------------------------
- *
- * OperSearch --
+ * OpSearch --
  *
  *      Performs a binary search on the array of command operation
  *      specifications to find a partial, anchored match for the
@@ -622,12 +654,12 @@ Blt_ConfigureValue(interp, tkwin, specs, widRec, option, flags)
  *----------------------------------------------------------------------
  */
 static int
-OperSearch(specArr, numSpecs, operation)
-    Blt_OperSpec specArr[];
+OpSearch(specArr, numSpecs, operation)
+    Blt_OpSpec specArr[];
     int numSpecs;
     char *operation;		/* Name of minor operation to search for */
 {
-    Blt_OperSpec *specPtr;
+    Blt_OpSpec *specPtr;
     char c;
     register int high, low, median;
     register int compare, length;
@@ -654,7 +686,7 @@ OperSearch(specArr, numSpecs, operation)
 	} else if (compare > 0) {
 	    low = median + 1;
 	} else {
-	    return (median);	/* Operation found. */
+	    return (median);	/* Op found. */
 	}
     }
     return -1;			/* Can't find operation */
@@ -677,17 +709,17 @@ OperSearch(specArr, numSpecs, operation)
  *
  *----------------------------------------------------------------------
  */
-Blt_OperProc
+Blt_Operation
 Blt_LookupOperation(interp, numSpecs, specArr, argIndex, numArgs, argArr)
     Tcl_Interp *interp;		/* Interpreter to report errors to */
     int numSpecs;		/* Number of specifications in array */
-    Blt_OperSpec specArr[];	/* Operation specification array */
-    Blt_OperIndex argIndex;	/* Index of the operation name argument */
+    Blt_OpSpec specArr[];	/* Operation specification array */
+    Blt_OpIndex argIndex;	/* Index of the operation name argument */
     int numArgs;		/* Number of arguments in the argument vector.
 				 * This includes any prefixed arguments */
     char *argArr[];		/* Argument vector */
 {
-    Blt_OperSpec *specPtr;
+    Blt_OpSpec *specPtr;
     char *string;
     register int i;
     register int specIndex;
@@ -695,8 +727,7 @@ Blt_LookupOperation(interp, numSpecs, specArr, argIndex, numArgs, argArr)
     if (numArgs <= argIndex) {	/* No operation argument */
 	Tcl_AppendResult(interp, "wrong # args: ", (char *)NULL);
       usage:
-	Tcl_AppendResult(interp, "should be one of...",
-	    (char *)NULL);
+	Tcl_AppendResult(interp, "should be one of...", (char *)NULL);
 	for (specIndex = 0; specIndex < numSpecs; specIndex++) {
 	    Tcl_AppendResult(interp, "\n  ", (char *)NULL);
 	    for (i = 0; i < argIndex; i++) {
@@ -709,7 +740,7 @@ Blt_LookupOperation(interp, numSpecs, specArr, argIndex, numArgs, argArr)
 	return NULL;
     }
     string = argArr[argIndex];
-    specIndex = OperSearch(specArr, numSpecs, string);
+    specIndex = OpSearch(specArr, numSpecs, string);
 
     if (specIndex == -2) {
 	char c;
@@ -755,7 +786,6 @@ Blt_LookupOperation(interp, numSpecs, specArr, argIndex, numArgs, argArr)
     return (specPtr->proc);
 }
 
-
 #ifndef HAVE_STRDUP
 /*
  *----------------------------------------------------------------------
@@ -784,6 +814,7 @@ strdup(string)
 
 #endif /*HAVE_STRDUP*/
 
+#ifdef notdef
 #ifndef HAVE_STRCASECMP
 
 static unsigned char lcase[] =
@@ -894,10 +925,12 @@ strncasecmp(str1, str2, length)
 #endif
 
 #endif /*HAVE_STRCASECMP*/
+#endif /* notdef */
+
 /*
  *----------------------------------------------------------------------
  *
- * Blt_AppendDouble --
+ * Blt_AppendDoubleElement --
  *
  *      Convenience routine to append a double precision value to
  * 	interp->result as a separate element.
@@ -912,7 +945,7 @@ strncasecmp(str1, str2, length)
  *----------------------------------------------------------------------
  */
 void
-Blt_AppendDouble(interp, value)
+Blt_AppendDoubleElement(interp, value)
     Tcl_Interp *interp;		/* Interpreter to append numeric string */
     double value;
 {
@@ -925,7 +958,7 @@ Blt_AppendDouble(interp, value)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_AppendInt --
+ * Blt_AppendIntElement --
  *
  *      Convenience routine to append an integer value to interp->result
  *	as a separate list element.
@@ -939,20 +972,79 @@ Blt_AppendDouble(interp, value)
  *----------------------------------------------------------------------
  */
 void
-Blt_AppendInt(interp, value)
+Blt_AppendIntElement(interp, value)
     Tcl_Interp *interp;		/* Interpreter to append numeric string */
     int value;
 {
-    char string[TCL_DOUBLE_SPACE + 1];
+    char string[200];
 
     sprintf(string, "%d", value);
     Tcl_AppendElement(interp, string);
 }
 
+#if (TCL_MAJOR_VERSION >= 8)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_GetUnsharedGC --
+ * Blt_ExprDouble --
+ *
+ *      Same functionality as pre-8.0 Tcl_ExprDouble.
+ *
+ * Results:
+ *      A standard Tcl result.
+ *
+ * Side effects:
+ *      The interpreter result is cleared.
+ *
+ *----------------------------------------------------------------------
+ */
+int
+Blt_ExprDouble(interp, string, valuePtr)
+    Tcl_Interp *interp;		/* Interpreter to append numeric string */
+    char *string;
+    double *valuePtr;
+{
+    if (Tcl_ExprDouble(interp, string, valuePtr) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    Tcl_ResetResult(interp);
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_ExprLong --
+ *
+ *      Same functionality as pre-8.0 Tcl_ExprLong.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      The interpreter result is cleared.
+ *
+ *----------------------------------------------------------------------
+ */
+int
+Blt_ExprLong(interp, string, valuePtr)
+    Tcl_Interp *interp;		/* Interpreter to append numeric string */
+    char *string;
+    long *valuePtr;
+{
+    if (Tcl_ExprLong(interp, string, valuePtr) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    Tcl_ResetResult(interp);
+    return TCL_OK;
+}
+#endif
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_GetPrivateGCFromDrawable --
  *
  *      Like Tk_GetGC, but doesn't share the GC with any other widget.
  *	This is needed because the certain GC parameters (like dashes)
@@ -965,32 +1057,456 @@ Blt_AppendInt(interp, value)
  *----------------------------------------------------------------------
  */
 GC
-Blt_GetUnsharedGC(tkwin, gcMask, valuePtr)
+Blt_GetPrivateGCFromDrawable(tkwin, drawable, gcMask, valuePtr)
+    Tk_Window tkwin;
+    Drawable drawable;
+    unsigned long gcMask;
+    XGCValues *valuePtr;
+{
+    Pixmap pixmap;
+    GC newGC;
+
+    pixmap = None;
+    if (drawable == None) {
+	Drawable root;
+	int depth;
+
+	root = RootWindow(Tk_Display(tkwin), Tk_ScreenNumber(tkwin));
+	depth = Tk_Depth(tkwin);
+
+	if (depth == DefaultDepth(Tk_Display(tkwin), Tk_ScreenNumber(tkwin))) {
+	    drawable = root;
+	} else {
+	    pixmap = Tk_GetPixmap(Tk_Display(tkwin), root, 1, 1, depth);
+	    drawable = pixmap;
+	}
+    }
+    newGC = XCreateGC(Tk_Display(tkwin), drawable, gcMask, valuePtr);
+    if (pixmap != None) {
+	Tk_FreePixmap(Tk_Display(tkwin), pixmap);
+    }
+    return (newGC);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_GetPrivateGC --
+ *
+ *      Like Tk_GetGC, but doesn't share the GC with any other widget.
+ *	This is needed because the certain GC parameters (like dashes)
+ *	can not be set via XCreateGC, therefore there is no way for
+ *	Tk's hashing mechanism to recognize that two such GCs differ.
+ *
+ * Results:
+ *      A new GC is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+GC
+Blt_GetPrivateGC(tkwin, gcMask, valuePtr)
     Tk_Window tkwin;
     unsigned long gcMask;
     XGCValues *valuePtr;
 {
-    Drawable drawable, root;
-    int isPixmap;
-    int depth, defaultDepth;
-    GC newGC;
-
-    isPixmap = FALSE;
-    defaultDepth = DefaultDepth(Tk_Display(tkwin), Tk_ScreenNumber(tkwin));
-    root = RootWindow(Tk_Display(tkwin), Tk_ScreenNumber(tkwin));
-    depth = Tk_Depth(tkwin);
-
-    if (Tk_WindowId(tkwin) != None) {
-	drawable = Tk_WindowId(tkwin);
-    } else if (depth == defaultDepth) {
-	drawable = root;
-    } else {
-	drawable = Tk_GetPixmap(Tk_Display(tkwin), root, 1, 1, depth);
-	isPixmap = TRUE;
-    }
-    newGC = XCreateGC(Tk_Display(tkwin), drawable, gcMask, valuePtr);
-    if (isPixmap) {
-	Tk_FreePixmap(Tk_Display(tkwin), drawable);
-    }
-    return (newGC);
+    return Blt_GetPrivateGCFromDrawable(tkwin, Tk_WindowId(tkwin), gcMask, 
+	valuePtr);
 }
+
+void
+Blt_FreePrivateGC(display, gc)
+    Display *display;
+    GC gc;
+{
+    Tk_FreeXId(display, (XID) XGContextFromGC(gc));
+    XFreeGC(display, gc);
+}
+
+/* ARGSUSED */
+static int
+XGetImageErrorProc(clientData, errEventPtr)
+    ClientData clientData;
+    XErrorEvent *errEventPtr;
+{
+    int *errorPtr = (int *)clientData;
+
+    *errorPtr = TCL_ERROR;
+    return 0;
+}
+
+int
+Blt_GetSnapshot(tkwin, drawable, width, height, colorTablePtr, colorPtrPtr, 
+	dataPtrPtr)
+    Tk_Window tkwin;
+    Drawable drawable;
+    int width, height;
+    Tcl_HashTable *colorTablePtr;
+    XColor **colorPtrPtr;
+    unsigned long **dataPtrPtr;
+{
+    XImage *imagePtr;
+    XColor *colorPtr, *colorArr;
+    unsigned long *dataPtr, *dataArr;
+    Tcl_HashEntry *hPtr;
+    Tcl_HashSearch cursor;
+    int numColors;
+    unsigned long pixel;
+    unsigned int numPixels;
+    int x, y;
+    int isNew;
+    int result = TCL_OK;
+    Tk_ErrorHandler errHandler;
+
+    errHandler = Tk_CreateErrorHandler(Tk_Display(tkwin), BadMatch,
+	X_GetImage, -1, XGetImageErrorProc, (ClientData)&result);
+    imagePtr = XGetImage(Tk_Display(tkwin), drawable, 0, 0, width, height,
+	AllPlanes, ZPixmap);
+    Tk_DeleteErrorHandler(errHandler);
+    XSync(Tk_Display(tkwin), False);
+    if (result != TCL_OK) {
+	return -1;
+    }
+
+    /*
+     * First allocate an array of pixels big enough to hold the image.
+     */
+    numPixels = width * height;
+    dataArr = (unsigned long *) malloc(sizeof(unsigned long) * numPixels);
+    assert(dataArr);
+    /*
+     * Fill the array with each pixel of the image. At the same time, build 
+     * up a hashtable of the pixels used.
+     */
+
+    Tcl_InitHashTable(colorTablePtr, TCL_ONE_WORD_KEYS);
+    dataPtr = dataArr;
+    for (y = 0; y < height; y++) {
+	for (x = 0; x < width; x++) {
+	    pixel = XGetPixel(imagePtr, x, y);
+	    hPtr = Tcl_CreateHashEntry(colorTablePtr, (char *)pixel, &isNew);
+	    if (isNew) {
+		Tcl_SetHashValue(hPtr, (char *)pixel);
+	    }
+	    *dataPtr++  = pixel;
+	}
+    }
+    XDestroyImage(imagePtr);
+
+    /* 
+     * Convert the hashtable of pixels into an array of XColors so that we 
+     * can call XQueryColors with it. XQueryColors will convert the pixels 
+     * into their RGB values.
+     */
+    numColors = Blt_ListGetLength(colorTablePtr);
+    colorArr = (XColor *)malloc(sizeof(XColor) * numColors);
+
+    colorPtr = colorArr;
+    for (hPtr = Tcl_FirstHashEntry(colorTablePtr, &cursor); hPtr != NULL;
+	hPtr = Tcl_NextHashEntry(&cursor)) {
+	colorPtr->pixel = (unsigned long)Tcl_GetHashValue(hPtr);
+	Tcl_SetHashValue(hPtr, (char *)colorPtr);
+	colorPtr++;
+    }
+    XQueryColors(Tk_Display(tkwin), Tk_Colormap(tkwin), colorArr, numColors);
+
+    *dataPtrPtr = dataArr;
+    *colorPtrPtr = colorArr;
+    return numPixels;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_SnapPhoto --
+ *
+ *      Takes a snapshot of an X drawable (pixmap or window) and 
+ *	writes it to an existing Tk_Image. 
+ *
+ *	The trick here is to efficiently convert the pixel values
+ *	(indices into the color table) into RGB color values.  In the
+ *	days of 8-bit displays, it was simpler to get RGB values for
+ *	all 256 indices into the colormap.  Instead we'll build a
+ *	hashtable of unique pixels and from that an array of pixels to
+ *	pass to XQueryColors.  
+ *
+ * Results:
+ *      A standard Tcl result.
+ *
+ * Side Effects:
+ *	The named Tk_Photo is updated with the XImage.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Blt_SnapPhoto(interp, tkwin, drawable, width, height, photoName)
+    Tcl_Interp *interp;
+    Tk_Window tkwin;
+    Drawable drawable;
+    int width, height;
+    char *photoName;
+{
+    Tk_PhotoHandle photo;	/* The photo image to write into. */
+    Tk_PhotoImageBlock block;
+    XColor *colorPtr, *colorArr;
+    unsigned long *dataArr;
+    PixelColor *pixelPtr, *pixelArr;
+    Tcl_HashTable colorTable;
+    Tcl_HashEntry *hPtr;
+    register int i;
+    int numPixels;
+
+    photo = Tk_FindPhoto(photoName);
+    if (photo == NULL) {
+	Tcl_AppendResult(interp, "image \"", photoName, "\" doesn't",
+	    " exist or is not a photo image", (char *)NULL);
+	return TCL_ERROR;
+    }
+
+    numPixels = Blt_GetSnapshot(tkwin, drawable, width, height, &colorTable,
+	&colorArr, &dataArr);
+    if (numPixels < 0) {
+	Tcl_AppendResult(interp, 
+		"can't grab window or pixmap (possibly obscured?)", (char *)NULL);
+	return TCL_ERROR;	/* Can't grab window image */
+    }
+    /*  
+     * Go through the array of pixels, replacing each pixel of the
+     * image with its RGB value.  
+     */
+    pixelPtr = pixelArr = (PixelColor *)malloc(sizeof(PixelColor) * numPixels);
+    for (i = 0; i < numPixels; i++) {
+	hPtr = Tcl_FindHashEntry(&colorTable, (char *)dataArr[i]);
+	colorPtr = (XColor *)Tcl_GetHashValue(hPtr);
+	pixelPtr->Red = (unsigned char)(colorPtr->red >> 8);
+	pixelPtr->Green = (unsigned char)(colorPtr->green >> 8);
+	pixelPtr->Blue = (unsigned char)(colorPtr->blue >> 8);
+	pixelPtr++;
+    }
+    free((char *)dataArr);
+    free((char *)colorArr);
+    Tcl_DeleteHashTable(&colorTable);
+
+    /* 
+     * Finally, we're ready to call the Tk_Photo routines. They'll take
+     * the RGB array we've processed to build the Tk image of the snapshot.
+     */
+    Tk_PhotoGetImage(photo, &block);
+    Tk_PhotoExpand(photo, width, height);
+    block.width = width;
+    block.height = height;
+    block.pixelSize = sizeof(PixelColor);
+    block.pitch = sizeof(PixelColor) * width;
+    block.offset[0] = Tk_Offset(RGB, red);
+    block.offset[1] = Tk_Offset(RGB, green);
+    block.offset[2] = Tk_Offset(RGB, blue);
+
+    block.pixelPtr = (unsigned char *)pixelArr;
+    Tk_PhotoPutBlock(photo, &block, 0, 0, width, height);
+    free((char *)block.pixelPtr);
+
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_FindChild --
+ *
+ *      Searches the parent for a named child window.
+ *
+ *	Oddly enough, the isn't in Tk's C API, but only in it's Tcl API.
+ *	It's easy enough to do here, if you peek into the Tk_FakeWin
+ *	structure.
+ *
+ * Results:
+ *      The child Tk_Window. If the named child can't be found, NULL
+ *	is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#define Tk_FirstChild(tkwin)	((Tk_Window)(((Tk_FakeWin *) (tkwin))->dummy2))
+#define Tk_NextChild(tkwin)	((Tk_Window)(((Tk_FakeWin *) (tkwin))->dummy4))
+
+
+/*LINTLIBRARY*/
+Tk_Window
+Blt_FindChild(parent, name)
+    Tk_Window parent;
+    char *name;
+{
+    register Tk_Window tkwin;
+
+    for (tkwin = Tk_FirstChild(parent); tkwin != NULL; 
+	 tkwin = Tk_NextChild(tkwin)) {
+	if (strcmp(name, Tk_Name(tkwin)) == 0) {
+	    return (tkwin);
+	}
+    }
+    return NULL;
+}
+    
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_FindChild --
+ *
+ *      Searches the parent for a named child window.
+ *
+ *	Oddly enough, the isn't in Tk's C API, but only in it's Tcl API.
+ *	It's easy enough to do here, if you peek into the Tk_FakeWin
+ *	structure.
+ *
+ * Results:
+ *      The child Tk_Window. If the named child can't be found, NULL
+ *	is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+
+#define Tk_FirstChild(tkwin)	((Tk_Window)(((Tk_FakeWin *) (tkwin))->dummy2))
+#define Tk_NextChild(tkwin)	((Tk_Window)(((Tk_FakeWin *) (tkwin))->dummy4))
+    
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_Toplevel --
+ *
+ *      Climbs up the widget hierarchy to find the top level window of
+ *      the window given.
+ *
+ * Results:
+ *      Returns the Tk_Window of the toplevel widget. 
+ *
+ *----------------------------------------------------------------------
+ */
+Tk_Window
+Blt_Toplevel(tkwin)
+    register Tk_Window tkwin;
+{
+    while (!Tk_IsTopLevel(tkwin)) {
+	tkwin = Tk_Parent(tkwin);
+    }
+    return tkwin;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_XWindowId --
+ *
+ *      Returns the XID for the Tk_Window given.  Starting in Tk 8.0, 
+ *	the toplevel widgets are wrapped by another window.  Currently
+ *	There's no way to get at that window, other than what is done
+ *	here.  Query the X window hierarchy and grab the parent.
+ *
+ * Results:
+ *      Returns the X Window ID of the widget.  If it's a toplevel, then
+ *	the XID of the wrapper is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+Window
+Blt_XWindowId(tkwin)
+    Tk_Window tkwin;
+{
+    Window window;
+
+    window = Tk_WindowId(tkwin);
+#if (TK_MAJOR_VERSION >= 8)
+    if (Tk_IsTopLevel(tkwin)) {
+	Window root, parent;
+	Window *childrenPtr;
+	unsigned int numChildren;
+
+	parent = None;
+	if (XQueryTree(Tk_Display(tkwin), Tk_WindowId(tkwin), &root, &parent, 
+		       &childrenPtr, &numChildren) > 0) {
+	    XFree(childrenPtr);
+	    window = parent;
+	}
+    }
+#endif /* TK_MAJOR_VERSION >= 8 */
+    return window;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_ConfigureWidgetComponent --
+ *
+ *      Configures a component of a widget.  This is useful for
+ *	multiple components which aren't uniquely identified by
+ *	an Tk_Window (not like a widget).
+ *
+ *	This is really a dumb hack to work around the limitations of
+ *	the Tk resource database.  It creates a temporary window,
+ *	needed to call Tk_ConfigureWidget, by the name of the
+ *	component.  This lets us, for example, set resources for axes
+ *	of the graph widget. The graph really has only one window, but
+ *	its convenient to specify components in a hierarchy of options.
+ *
+ *		*graph.x.logScale yes
+ *		*graph.Axis.logScale yes
+ *		*graph.temperature.scaleSymbols yes
+ *		*graph.Element.scaleSymbols yes
+ *
+ * Results:
+ *      A standard Tcl result.
+ *
+ * Side Effects:
+ *	Temporary window is generated merely to call with 
+ *	Tk_ConfigureWidget.
+ *
+ *---------------------------------------------------------------------- */
+int
+Blt_ConfigureWidgetComponent(interp, parent, name, class, specsPtr, argc, argv, 
+	widgRec, flags)
+    Tcl_Interp *interp;
+    Tk_Window parent;		/* Window to associate with component */
+    char name[];		/* Name of component */
+    char class[];
+    Tk_ConfigSpec *specsPtr;
+    int argc;
+    char *argv[];
+    char *widgRec;
+    int flags;
+{
+    Tk_Window tkwin;
+    int result;
+    char *tempName;
+    int temporary = 0;
+
+    tempName = strdup(name);
+
+    /* Window name can't start with an upper case letter */
+    tempName[0] = tolower(name[0]);
+
+    /* 
+     * Create component if a child window by the component's name
+     * doesn't already exist. 
+     */
+    tkwin = Blt_FindChild(parent, tempName);
+    if (tkwin == NULL) {
+	tkwin = Tk_CreateWindow(interp, parent, tempName, (char *)NULL);
+	temporary = 1;
+    }
+    assert(Tk_Depth(tkwin) == Tk_Depth(parent));
+    free(tempName);
+
+    if (tkwin == NULL) {
+	return TCL_ERROR;
+    }
+    Tk_SetClass(tkwin, class);
+    result = Tk_ConfigureWidget(interp, tkwin, specsPtr, argc, argv, widgRec,
+	flags);
+    if (temporary) {
+	Tk_DestroyWindow(tkwin);
+    }
+    return (result);
+}
+
+
+
