@@ -11,13 +11,12 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkButton.c 1.128 96/03/01 17:34:49
  */
 
-static char sccsid[] = "@(#) tkButton.c 1.121 95/09/25 14:56:29";
-
 #include "bltInt.h"
 
-#if (TK_MAJOR_VERSION == 4)
 extern Tk_CustomOption bltTileOption;
 /*
  * The definitions below provide symbolic names for the default colors.
@@ -49,6 +48,7 @@ static Tk_Uid tkNormalUid, tkActiveUid, tkDisabledUid;
 #define DEF_BUTTON_BORDER_WIDTH		"2"
 #define DEF_BUTTON_CURSOR		""
 #define DEF_BUTTON_COMMAND		""
+#define DEF_BUTTON_DEFAULT              "0"
 #define DEF_BUTTON_DISABLED_FG_COLOR	STD_DISABLE_FG_COLOR
 #define DEF_BUTTON_DISABLED_FG_MONO	""
 #define DEF_BUTTON_FG			BLACK
@@ -153,7 +153,7 @@ typedef struct {
 				 * Indicates how much interior stuff must
 				 * be offset from outside edges to leave
 				 * room for borders. */
-    XFontStruct *fontPtr;	/* Information about text font, or NULL. */
+    Tk_Font font;		/* Information about text font, or NULL. */
     XColor *normalFg;		/* Foreground color in normal mode. */
     XColor *activeFg;		/* Foreground color in active mode.  NULL
 				 * means use normalFg instead. */
@@ -199,10 +199,15 @@ typedef struct {
 				 * in pixels. */
     int textHeight;		/* Height needed to display text as requested,
 				 * in pixels. */
+#if (TK_MAJOR_VERSION >= 8)
+    Tk_TextLayout textLayout;	/* Saved text layout information. */
+#endif
     int indicatorSpace;		/* Horizontal space (in pixels) allocated for
 				 * display of indicator. */
     int indicatorDiameter;	/* Diameter of indicator, in pixels. */
 
+    int isDefault;		/* Used in 8.0 (not here)  */
+
     /*
      * For check and radio buttons, the fields below are used
      * to manage the variable indicating the button's state.
@@ -223,7 +228,7 @@ typedef struct {
      * Miscellaneous information:
      */
 
-    Cursor cursor;		/* Current cursor for window, or None. */
+    Tk_Cursor cursor;		/* Current cursor for window, or None. */
     char *takeFocus;		/* Value of -takefocus option;  not used in
 				 * the C code, but used by keyboard traversal
 				 * scripts.  Malloc'ed, but may be NULL. */
@@ -234,7 +239,6 @@ typedef struct {
 				 * definitions. */
     Blt_Tile tile, activeTile;
     GC tileGC, activeTileGC;
-
 } Button;
 
 /*
@@ -333,6 +337,8 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_BUTTON_CURSOR, Tk_Offset(Button, cursor),
 	ALL_MASK|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_BOOLEAN, "-default", "default", "Default",
+        DEF_BUTTON_DEFAULT, Tk_Offset(Button, isDefault), BUTTON_MASK},
     {TK_CONFIG_COLOR, "-disabledforeground", "disabledForeground",
 	"DisabledForeground", DEF_BUTTON_DISABLED_FG_COLOR,
 	Tk_Offset(Button, disabledFg), BUTTON_MASK|CHECK_BUTTON_MASK
@@ -344,7 +350,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, ALL_MASK},
     {TK_CONFIG_FONT, "-font", "font", "Font",
-	DEF_BUTTON_FONT, Tk_Offset(Button, fontPtr),
+	DEF_BUTTON_FONT, Tk_Offset(Button, font),
 	ALL_MASK},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_BUTTON_FG, Tk_Offset(Button, normalFg), ALL_MASK},
@@ -481,14 +487,86 @@ static void		ComputeButtonGeometry _ANSI_ARGS_((Button *butPtr));
 static int		ConfigureButton _ANSI_ARGS_((Tcl_Interp *interp,
 			    Button *butPtr, int argc, char **argv,
 			    int flags));
-static void		DestroyButton _ANSI_ARGS_((ClientData clientData));
+static void		DestroyButton _ANSI_ARGS_((Button *butPtr));
 static void		DisplayButton _ANSI_ARGS_((ClientData clientData));
 static int		InvokeButton  _ANSI_ARGS_((Button *butPtr));
+
+#if (TK_MAJOR_VERSION < 8)
+/*
+ *---------------------------------------------------------------------------
+ *
+ * TkComputeAnchor --
+ *
+ *	Determine where to place a rectangle so that it will be properly
+ *	anchored with respect to the given window.  Used by widgets
+ *	to align a box of text inside a window.  When anchoring with
+ *	respect to one of the sides, the rectangle be placed inside of
+ *	the internal border of the window.
+ *
+ * Results:
+ *	*xPtr and *yPtr set to the upper-left corner of the rectangle
+ *	anchored in the window.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+static void
+TkComputeAnchor(anchor, tkwin, padX, padY, innerWidth, innerHeight, xPtr, yPtr)
+    Tk_Anchor anchor;		/* Desired anchor. */
+    Tk_Window tkwin;		/* Anchored with respect to this window. */
+    int padX, padY;		/* Use this extra padding inside window, in
+				 * addition to the internal border. */
+    int innerWidth, innerHeight;/* Size of rectangle to anchor in window. */
+    int *xPtr, *yPtr;		/* Returns upper-left corner of anchored
+				 * rectangle. */
+{
+    switch (anchor) {
+	case TK_ANCHOR_NW:
+	case TK_ANCHOR_W:
+	case TK_ANCHOR_SW:
+	    *xPtr = Tk_InternalBorderWidth(tkwin) + padX;
+	    break;
+
+	case TK_ANCHOR_N:
+	case TK_ANCHOR_CENTER:
+	case TK_ANCHOR_S:
+	    *xPtr = (Tk_Width(tkwin) - innerWidth) / 2;
+	    break;
+
+	default:
+	    *xPtr = Tk_Width(tkwin) - (Tk_InternalBorderWidth(tkwin) + padX)
+		    - innerWidth;
+	    break;
+    }
+
+    switch (anchor) {
+	case TK_ANCHOR_NW:
+	case TK_ANCHOR_N:
+	case TK_ANCHOR_NE:
+	    *yPtr = Tk_InternalBorderWidth(tkwin) + padY;
+	    break;
+
+	case TK_ANCHOR_W:
+	case TK_ANCHOR_CENTER:
+	case TK_ANCHOR_E:
+	    *yPtr = (Tk_Height(tkwin) - innerHeight) / 2;
+	    break;
+
+	default:
+	    *yPtr = Tk_Height(tkwin) - Tk_InternalBorderWidth(tkwin) - padY
+		    - innerHeight;
+	    break;
+    }
+}
+#endif
+
 
 /*
  *--------------------------------------------------------------
  *
- * ButtonCmd, CheckbuttonCmd, LabelCmd, RadiobuttonCmd --
+ * Tk_ButtonCmd, Tk_CheckbuttonCmd, Tk_LabelCmd, Tk_RadiobuttonCmd --
  *
  *	These procedures are invoked to process the "button", "label",
  *	"radiobutton", and "checkbutton" Tcl commands.  See the
@@ -582,7 +660,7 @@ ButtonCreate(clientData, interp, argc, argv, type)
     Tk_Window new;
 
     if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args:  should be \"",
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
 		argv[0], " pathName ?options?\"", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -629,7 +707,7 @@ ButtonCreate(clientData, interp, argc, argv, type)
     butPtr->highlightBgColorPtr = NULL;
     butPtr->highlightColorPtr = NULL;
     butPtr->inset = 0;
-    butPtr->fontPtr = NULL;
+    butPtr->font = NULL;
     butPtr->normalFg = NULL;
     butPtr->activeFg = NULL;
     butPtr->disabledFg = NULL;
@@ -647,6 +725,9 @@ ButtonCreate(clientData, interp, argc, argv, type)
     butPtr->padY = 0;
     butPtr->anchor = TK_ANCHOR_CENTER;
     butPtr->justify = TK_JUSTIFY_CENTER;
+#if (TK_MAJOR_VERSION >= 8)
+    butPtr->textLayout = NULL;
+#endif
     butPtr->indicatorOn = 0;
     butPtr->selectBorder = NULL;
     butPtr->indicatorSpace = 0;
@@ -660,6 +741,7 @@ ButtonCreate(clientData, interp, argc, argv, type)
     butPtr->flags = 0;
     butPtr->tile = butPtr->activeTile = NULL;
     butPtr->tileGC = butPtr->activeTileGC = None;
+    butPtr->isDefault = 0;
 
     Tk_SetClass(new, classNames[type]);
     Tk_CreateEventHandler(butPtr->tkwin,
@@ -711,7 +793,7 @@ ButtonWidgetCmd(clientData, interp, argc, argv)
 		argv[0]);
 	return TCL_ERROR;
     }
-    Tk_Preserve((ClientData) butPtr);
+    Tcl_Preserve((ClientData) butPtr);
     c = argv[1][0];
     length = strlen(argv[1]);
     if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
@@ -776,14 +858,14 @@ ButtonWidgetCmd(clientData, interp, argc, argv)
 		DisplayButton((ClientData) butPtr);
 
 		/*
-		 * Special note:  must cancel any existing idle handler
+		 * Special note: must cancel any existing idle handler
 		 * for DisplayButton;  it's no longer needed, and DisplayButton
 		 * cleared the REDRAW_PENDING flag.
 		 */
 
-		Tk_CancelIdleCall(DisplayButton, (ClientData) butPtr);
+		Tcl_CancelIdleCall(DisplayButton, (ClientData) butPtr);
 		XFlush(butPtr->display);
-		Tk_Sleep(50);
+		Tcl_Sleep(50);
 	    }
 	}
     } else if ((c == 'i') && (strncmp(argv[1], "invoke", length) == 0)
@@ -830,15 +912,15 @@ ButtonWidgetCmd(clientData, interp, argc, argv)
 	}
     } else {
 	sprintf(interp->result,
-		"bad option \"%.50s\":  must be %s", argv[1],
+		"bad option \"%.50s\": must be %s", argv[1],
 		optionStrings[butPtr->type]);
 	goto error;
     }
-    Tk_Release((ClientData) butPtr);
+    Tcl_Release((ClientData) butPtr);
     return result;
 
     error:
-    Tk_Release((ClientData) butPtr);
+    Tcl_Release((ClientData) butPtr);
     return TCL_ERROR;
 }
 
@@ -847,7 +929,7 @@ ButtonWidgetCmd(clientData, interp, argc, argv)
  *
  * DestroyButton --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a button at a safe time
  *	(when no-one is using it anymore).
  *
@@ -861,11 +943,9 @@ ButtonWidgetCmd(clientData, interp, argc, argv)
  */
 
 static void
-DestroyButton(clientData)
-    ClientData clientData;		/* Info about entry widget. */
+DestroyButton(butPtr)
+    Button *butPtr;		/* Info about button widget. */
 {
-    register Button *butPtr = (Button *) clientData;
-
     /*
      * Free up all the stuff that requires special handling, then
      * let Tk_FreeOptions handle all the standard option-related
@@ -915,23 +995,28 @@ DestroyButton(clientData)
     if (butPtr->tileGC != None) {
 	Tk_FreeGC(butPtr->display, butPtr->tileGC);
     }
+#if (TK_MAJOR_VERSION >= 8)
+    Tk_FreeTextLayout(butPtr->textLayout);
+#endif
     Tk_FreeOptions(configSpecs, (char *) butPtr, butPtr->display,
 	    configFlags[butPtr->type]);
-    ckfree((char *) butPtr);
+    Tcl_EventuallyFree((ClientData)butPtr, TCL_DYNAMIC);
 }
-
 
 static void
-TileChangedProc(butPtr, tile, gcPtr)
-    Button *butPtr;
+TileChangedProc(clientData, tile)
+    ClientData clientData;
     Blt_Tile tile;
-    GC *gcPtr;
-
 {
+    Button *butPtr = (Button *)clientData;
+
     if (butPtr->tkwin != NULL) {
 	GC newGC;
 	Pixmap pixmap;
+	GC *gcPtr;
 
+	gcPtr = (tile == butPtr->tile) 
+	    ? &(butPtr->tileGC) : &(butPtr->activeTileGC);
 	newGC = NULL;
 	pixmap = Blt_PixmapOfTile(tile);
 	if (pixmap != None) {
@@ -979,6 +1064,7 @@ TileChangedProc(butPtr, tile, gcPtr)
  *
  *----------------------------------------------------------------------
  */
+
 static int
 ConfigureButton(interp, butPtr, argc, argv, flags)
     Tcl_Interp *interp;		/* Used for error reporting. */
@@ -990,7 +1076,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 {
     XGCValues gcValues;
     GC newGC;
-    unsigned long mask, clipMask;
+    unsigned long mask;
     Tk_Image image;
 
     /*
@@ -1026,7 +1112,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 	if ((butPtr->state != tkNormalUid) && (butPtr->state != tkActiveUid)
 		&& (butPtr->state != tkDisabledUid)) {
 	    Tcl_AppendResult(interp, "bad state value \"", butPtr->state,
-		    "\":  must be normal, active, or disabled", (char *) NULL);
+		    "\": must be normal, active, or disabled", (char *) NULL);
 	    butPtr->state = tkNormalUid;
 	    return TCL_ERROR;
 	}
@@ -1036,15 +1122,9 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 	butPtr->highlightWidth = 0;
     }
 
-    gcValues.font = butPtr->fontPtr->fid;
+    gcValues.font = Tk_FontId(butPtr->font);
     gcValues.foreground = butPtr->normalFg->pixel;
     gcValues.background = Tk_3DBorderColor(butPtr->normalBorder)->pixel;
-    gcValues.clip_mask = butPtr->bitmap;
-    if ((butPtr->bitmap != None) && (butPtr->imageString == NULL)) {
-	clipMask = GCClipMask;
-    } else {
-	clipMask = 0;
-    }
 
     if (butPtr->tile != NULL) {
 	Pixmap pixmap;
@@ -1055,7 +1135,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 	    unsigned long gcMask;
 
 	    Blt_SetTileChangedProc(butPtr->tile, TileChangedProc, 
-				   (ClientData)butPtr, &(butPtr->tileGC));
+				   (ClientData)butPtr);
 	    gcMask = (GCTile | GCFillStyle);
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
@@ -1076,7 +1156,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 	    unsigned long gcMask;
 
 	    Blt_SetTileChangedProc(butPtr->activeTile, TileChangedProc, 
-				   (ClientData)butPtr, &(butPtr->activeTileGC));
+				   (ClientData)butPtr);
 	    gcMask = (GCTile | GCFillStyle);
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
@@ -1088,6 +1168,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 	butPtr->activeTileGC = newGC;
     }
 
+    
     /*
      * Note: GraphicsExpose events are disabled in normalTextGC because it's
      * used to copy stuff from an off-screen pixmap onto the screen (we know
@@ -1096,7 +1177,7 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
 
     gcValues.graphics_exposures = False;
     newGC = Tk_GetGC(butPtr->tkwin,
-	    clipMask|GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+	    GCForeground|GCBackground|GCFont|GCGraphicsExposures,
 	    &gcValues);
     if (butPtr->normalTextGC != None) {
 	Tk_FreeGC(butPtr->display, butPtr->normalTextGC);
@@ -1104,11 +1185,11 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
     butPtr->normalTextGC = newGC;
 
     if (butPtr->activeFg != NULL) {
-	gcValues.font = butPtr->fontPtr->fid;
+	gcValues.font = Tk_FontId(butPtr->font);
 	gcValues.foreground = butPtr->activeFg->pixel;
 	gcValues.background = Tk_3DBorderColor(butPtr->activeBorder)->pixel;
 	newGC = Tk_GetGC(butPtr->tkwin,
-		clipMask|GCForeground|GCBackground|GCFont, &gcValues);
+		GCForeground|GCBackground|GCFont, &gcValues);
 	if (butPtr->activeTextGC != None) {
 	    Tk_FreeGC(butPtr->display, butPtr->activeTextGC);
 	}
@@ -1116,11 +1197,11 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
     }
 
     if (butPtr->type != TYPE_LABEL) {
-	gcValues.font = butPtr->fontPtr->fid;
+	gcValues.font = Tk_FontId(butPtr->font);
 	gcValues.background = Tk_3DBorderColor(butPtr->normalBorder)->pixel;
 	if ((butPtr->disabledFg != NULL) && (butPtr->imageString == NULL)) {
 	    gcValues.foreground = butPtr->disabledFg->pixel;
-	    mask = clipMask|GCForeground|GCBackground|GCFont;
+	    mask = GCForeground|GCBackground|GCFont;
 	} else {
 	    gcValues.foreground = gcValues.background;
 	    if (butPtr->gray == None) {
@@ -1278,13 +1359,419 @@ ConfigureButton(interp, butPtr, argc, argv, flags)
      */
 
     if (Tk_IsMapped(butPtr->tkwin) && !(butPtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	butPtr->flags |= REDRAW_PENDING;
     }
 
     return TCL_OK;
 }
+/*
+ *----------------------------------------------------------------------
+ *
+ * DisplayButton --
+ *
+ *	This procedure is invoked to display a button widget.  It is
+ *	normally invoked as an idle handler.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Commands are output to X to display the button in its
+ *	current mode.  The REDRAW_PENDING flag is cleared.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+DisplayButton(clientData)
+    ClientData clientData;	/* Information about widget. */
+{
+    register Button *butPtr = (Button *) clientData;
+    GC gc;
+    Tk_3DBorder border;
+    Pixmap pixmap;
+    int x = 0;			/* Initialization only needed to stop
+				 * compiler warning. */
+    int y, relief;
+    register Tk_Window tkwin = butPtr->tkwin;
+    int width, height;
+    int offset;			/* 0 means this is a label widget.  1 means
+				 * it is a flavor of button, so we offset
+				 * the text to make the button appear to
+				 * move up and down as the relief changes. */
+    GC tileGC;
+
+    butPtr->flags &= ~REDRAW_PENDING;
+    if ((butPtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
+	return;
+    }
+
+    tileGC = butPtr->tileGC;
+    border = butPtr->normalBorder;
+    if ((butPtr->state == tkDisabledUid) && (butPtr->disabledFg != NULL)) {
+	gc = butPtr->disabledGC;
+    } else if ((butPtr->state == tkActiveUid)
+	    && !Tk_StrictMotif(butPtr->tkwin)) {
+	gc = butPtr->activeTextGC;
+	border = butPtr->activeBorder;
+	tileGC = butPtr->activeTileGC;
+    } else {
+	gc = butPtr->normalTextGC;
+    }
+    if ((butPtr->flags & SELECTED) && (butPtr->state != tkActiveUid)
+	    && (butPtr->selectBorder != NULL) && !butPtr->indicatorOn) {
+	border = butPtr->selectBorder;
+    }
+
+    /*
+     * Override the relief specified for the button if this is a
+     * checkbutton or radiobutton and there's no indicator.
+     */
+
+    relief = butPtr->relief;
+    if ((butPtr->type >= TYPE_CHECK_BUTTON) && !butPtr->indicatorOn) {
+	relief = (butPtr->flags & SELECTED) ? TK_RELIEF_SUNKEN
+		: TK_RELIEF_RAISED;
+    }
+
+    offset = (butPtr->type == TYPE_BUTTON) && !Tk_StrictMotif(butPtr->tkwin);
+
+    /*
+     * In order to avoid screen flashes, this procedure redraws
+     * the button in a pixmap, then copies the pixmap to the
+     * screen in a single operation.  This means that there's no
+     * point in time where the on-sreen image has been cleared.
+     */
+
+    pixmap = Tk_GetPixmap(butPtr->display, Tk_WindowId(tkwin),
+	    Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
+    if (tileGC != NULL) {
+	Blt_SetTileOrigin(tkwin, tileGC, 0, 0);
+	XFillRectangle(butPtr->display, pixmap, tileGC, 0, 0, Tk_Width(tkwin), 
+		Tk_Height(tkwin));
+    } else {
+	Tk_Fill3DRectangle(tkwin, pixmap, border, 0, 0, Tk_Width(tkwin),
+	    Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
+    }
+
+    /*
+     * Display image or bitmap or text for button.
+     */
+
+    if (butPtr->image != None) {
+	Tk_SizeOfImage(butPtr->image, &width, &height);
+
+	imageOrBitmap:
+	TkComputeAnchor(butPtr->anchor, tkwin, 0, 0,
+		butPtr->indicatorSpace + width, height, &x, &y);
+	x += butPtr->indicatorSpace;
+
+	x += offset;
+	y += offset;
+	if (relief == TK_RELIEF_RAISED) {
+	    x -= offset;
+	    y -= offset;
+	} else if (relief == TK_RELIEF_SUNKEN) {
+	    x += offset;
+	    y += offset;
+	}
+	if (butPtr->image != NULL) {
+	    if ((butPtr->selectImage != NULL) && (butPtr->flags & SELECTED)) {
+		Tk_RedrawImage(butPtr->selectImage, 0, 0, width, height, pixmap,
+			x, y);
+	    } else {
+		Tk_RedrawImage(butPtr->image, 0, 0, width, height, pixmap,
+			x, y);
+	    }
+	} else {
+	    XSetClipOrigin(butPtr->display, gc, x, y);
+	    if (tileGC != NULL) {
+		XSetClipMask(butPtr->display, gc, butPtr->bitmap);
+	    }
+	    XCopyPlane(butPtr->display, butPtr->bitmap, pixmap, gc, 0, 0,
+		    (unsigned int) width, (unsigned int) height, x, y, 1);
+	    if (tileGC != NULL) {
+		XSetClipMask(butPtr->display, gc, None);
+	    }
+	    XSetClipOrigin(butPtr->display, gc, 0, 0);
+	}
+	y += height/2;
+    } else if (butPtr->bitmap != None) {
+	Tk_SizeOfBitmap(butPtr->display, butPtr->bitmap, &width, &height);
+	goto imageOrBitmap;
+    } else {
+	TkComputeAnchor(butPtr->anchor, tkwin, butPtr->padX, butPtr->padY,
+		butPtr->indicatorSpace + butPtr->textWidth, butPtr->textHeight,
+		&x, &y);
+
+	x += butPtr->indicatorSpace;
+
+	x += offset;
+	y += offset;
+	if (relief == TK_RELIEF_RAISED) {
+	    x -= offset;
+	    y -= offset;
+	} else if (relief == TK_RELIEF_SUNKEN) {
+	    x += offset;
+	    y += offset;
+	}
+#if (TK_MAJOR_VERSION >= 8)
+        Tk_DrawTextLayout(butPtr->display, pixmap, gc, butPtr->textLayout,
+                x, y, 0, -1);
+        Tk_UnderlineTextLayout(butPtr->display, pixmap, gc,
+                butPtr->textLayout, x, y, butPtr->underline);
+#else
+	TkDisplayText(butPtr->display, pixmap, butPtr->font,
+		butPtr->text, butPtr->numChars, x, y, butPtr->textWidth,
+		butPtr->justify, butPtr->underline, gc);
+#endif
+	y += butPtr->textHeight/2;
+    }
+
+    /*
+     * Draw the indicator for check buttons and radio buttons.  At this
+     * point x and y refer to the top-left corner of the text or image
+     * or bitmap.
+     */
+
+    if ((butPtr->type == TYPE_CHECK_BUTTON) && butPtr->indicatorOn) {
+	int dim;
+
+	dim = butPtr->indicatorDiameter;
+	x -= butPtr->indicatorSpace;
+	y -= dim/2;
+	if (dim > 2*butPtr->borderWidth) {
+	    Tk_Draw3DRectangle(tkwin, pixmap, border, x, y, dim, dim,
+		    butPtr->borderWidth, 
+		    (butPtr->flags & SELECTED) ? TK_RELIEF_SUNKEN :
+		    TK_RELIEF_RAISED);
+	    x += butPtr->borderWidth;
+	    y += butPtr->borderWidth;
+	    dim -= 2*butPtr->borderWidth;
+	    if (butPtr->flags & SELECTED) {
+		GC borderGC;
+
+		borderGC = Tk_3DBorderGC(tkwin,(butPtr->selectBorder != NULL)
+			? butPtr->selectBorder : butPtr->normalBorder,
+			TK_3D_FLAT_GC);
+		XFillRectangle(butPtr->display, pixmap, borderGC, x, y,
+			(unsigned int) dim, (unsigned int) dim);
+	    } else {
+		Tk_Fill3DRectangle(tkwin, pixmap, butPtr->normalBorder, x, y,
+			dim, dim, butPtr->borderWidth, TK_RELIEF_FLAT);
+	    }
+	}
+    } else if ((butPtr->type == TYPE_RADIO_BUTTON) && butPtr->indicatorOn) {
+	XPoint points[4];
+	int radius;
+
+	radius = butPtr->indicatorDiameter/2;
+	points[0].x = x - butPtr->indicatorSpace;
+	points[0].y = y;
+	points[1].x = points[0].x + radius;
+	points[1].y = points[0].y + radius;
+	points[2].x = points[1].x + radius;
+	points[2].y = points[0].y;
+	points[3].x = points[1].x;
+	points[3].y = points[0].y - radius;
+	if (butPtr->flags & SELECTED) {
+	    GC borderGC;
+
+	    borderGC = Tk_3DBorderGC(tkwin, (butPtr->selectBorder != NULL)
+		    ? butPtr->selectBorder : butPtr->normalBorder,
+		    TK_3D_FLAT_GC);
+	    XFillPolygon(butPtr->display, pixmap, borderGC, points, 4, Convex,
+		    CoordModeOrigin);
+	} else {
+	    Tk_Fill3DPolygon(tkwin, pixmap, butPtr->normalBorder, points,
+		    4, butPtr->borderWidth, TK_RELIEF_FLAT);
+	}
+	Tk_Draw3DPolygon(tkwin, pixmap, border, points, 4, butPtr->borderWidth,
+		(butPtr->flags & SELECTED) ? TK_RELIEF_SUNKEN :
+		TK_RELIEF_RAISED);
+    }
+
+    /*
+     * If the button is disabled with a stipple rather than a special
+     * foreground color, generate the stippled effect.  If the widget
+     * is selected and we use a different background color when selected,
+     * must temporarily modify the GC.
+     */
+
+    if ((butPtr->state == tkDisabledUid)
+	    && ((butPtr->disabledFg == NULL) || (butPtr->image != NULL))) {
+	if ((butPtr->flags & SELECTED) && !butPtr->indicatorOn
+		&& (butPtr->selectBorder != NULL)) {
+	    XSetForeground(butPtr->display, butPtr->disabledGC,
+		    Tk_3DBorderColor(butPtr->selectBorder)->pixel);
+	}
+	XFillRectangle(butPtr->display, pixmap, butPtr->disabledGC,
+		butPtr->inset, butPtr->inset,
+		(unsigned) (Tk_Width(tkwin) - 2*butPtr->inset),
+		(unsigned) (Tk_Height(tkwin) - 2*butPtr->inset));
+	if ((butPtr->flags & SELECTED) && !butPtr->indicatorOn
+		&& (butPtr->selectBorder != NULL)) {
+	    XSetForeground(butPtr->display, butPtr->disabledGC,
+		    Tk_3DBorderColor(butPtr->normalBorder)->pixel);
+	}
+    }
+
+    /*
+     * Draw the border and traversal highlight last.  This way, if the
+     * button's contents overflow they'll be covered up by the border.
+     */
+
+    if (relief != TK_RELIEF_FLAT) {
+	int inset = butPtr->highlightWidth;
+	if (butPtr->isDefault) {
+	    inset += 2;
+	    Tk_Draw3DRectangle(tkwin, pixmap, border, inset, inset,
+		    Tk_Width(tkwin) - 2*inset, Tk_Height(tkwin) - 2*inset,
+		    1, TK_RELIEF_SUNKEN);
+	    inset += 3;
+	}
+	Tk_Draw3DRectangle(tkwin, pixmap, border, inset, inset,
+		Tk_Width(tkwin) - 2*inset, Tk_Height(tkwin) - 2*inset,
+		butPtr->borderWidth, relief);
+    }
+    if (butPtr->highlightWidth != 0) {
+	GC highlightGC;
+
+	if (butPtr->flags & GOT_FOCUS) {
+	    highlightGC = Tk_GCForColor(butPtr->highlightColorPtr, pixmap);
+	} else {
+	    highlightGC = Tk_GCForColor(butPtr->highlightBgColorPtr, pixmap);
+	}
+	Tk_DrawFocusHighlight(tkwin, highlightGC, butPtr->highlightWidth, pixmap);
+    }
+
+    /*
+     * Copy the information from the off-screen pixmap onto the screen,
+     * then delete the pixmap.
+     */
+
+    XCopyArea(butPtr->display, pixmap, Tk_WindowId(tkwin),
+	    butPtr->copyGC, 0, 0, (unsigned) Tk_Width(tkwin),
+	    (unsigned) Tk_Height(tkwin), 0, 0);
+    Tk_FreePixmap(butPtr->display, pixmap);
+}
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ComputeButtonGeometry --
+ *
+ *	After changes in a button's text or bitmap, this procedure
+ *	recomputes the button's geometry and passes this information
+ *	along to the geometry manager for the window.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The button's window may change size.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+ComputeButtonGeometry(butPtr)
+    register Button *butPtr;	/* Button whose geometry may have changed. */
+{
+    int width, height;
+
+    if (butPtr->highlightWidth < 0) {
+	butPtr->highlightWidth = 0;
+    }
+    butPtr->inset = butPtr->highlightWidth + butPtr->borderWidth;
+
+    /*
+     * Leave room for the default ring if needed.
+     */
+
+    if (butPtr->isDefault) {
+	butPtr->inset += 5;
+    }
+    butPtr->indicatorSpace = 0;
+    if (butPtr->image != NULL) {
+	Tk_SizeOfImage(butPtr->image, &width, &height);
+	imageOrBitmap:
+	if (butPtr->width > 0) {
+	    width = butPtr->width;
+	}
+	if (butPtr->height > 0) {
+	    height = butPtr->height;
+	}
+	if ((butPtr->type >= TYPE_CHECK_BUTTON) && butPtr->indicatorOn) {
+	    butPtr->indicatorSpace = height;
+	    if (butPtr->type == TYPE_CHECK_BUTTON) {
+		butPtr->indicatorDiameter = (65*height)/100;
+	    } else {
+		butPtr->indicatorDiameter = (75*height)/100;
+	    }
+	}
+    } else if (butPtr->bitmap != None) {
+	Tk_SizeOfBitmap(butPtr->display, butPtr->bitmap, &width, &height);
+	goto imageOrBitmap;
+    } else {
+	int avgWidth;
+	Tk_FontMetrics fm;
+
+#if (TK_MAJOR_VERSION >= 8)
+	Tk_FreeTextLayout(butPtr->textLayout);
+	butPtr->textLayout = Tk_ComputeTextLayout(butPtr->font,
+		butPtr->text, -1, butPtr->wrapLength, butPtr->justify, 0,
+		&butPtr->textWidth, &butPtr->textHeight);
+#else
+	butPtr->numChars = strlen(butPtr->text);
+	TkComputeTextGeometry(butPtr->font, butPtr->text,
+		butPtr->numChars, butPtr->wrapLength, &butPtr->textWidth,
+		&butPtr->textHeight);
+#endif
+	width = butPtr->textWidth;
+	height = butPtr->textHeight;
+	avgWidth = Tk_TextWidth(butPtr->font, "0", 1);
+	Tk_GetFontMetrics(butPtr->font, &fm);
+
+	if (butPtr->width > 0) {
+	    width = butPtr->width * avgWidth;
+	}
+	if (butPtr->height > 0) {
+	    height = butPtr->height * fm.linespace;
+	}
+	if ((butPtr->type >= TYPE_CHECK_BUTTON) && butPtr->indicatorOn) {
+	    butPtr->indicatorDiameter = fm.linespace;
+	    if (butPtr->type == TYPE_CHECK_BUTTON) {
+		butPtr->indicatorDiameter = (80*butPtr->indicatorDiameter)/100;
+	    }
+	    butPtr->indicatorSpace = butPtr->indicatorDiameter + avgWidth;
+	}
+    }
+
+    /*
+     * When issuing the geometry request, add extra space for the indicator,
+     * if any, and for the border and padding, plus two extra pixels so the
+     * display can be offset by 1 pixel in either direction for the raised
+     * or lowered effect.
+     */
+
+    if ((butPtr->image == NULL) && (butPtr->bitmap == None)) {
+	width += 2*butPtr->padX;
+	height += 2*butPtr->padY;
+    }
+    if ((butPtr->type == TYPE_BUTTON) && !Tk_StrictMotif(butPtr->tkwin)) {
+	width += 2;
+	height += 2;
+    }
+    Tk_GeometryRequest(butPtr->tkwin, (int) (width + butPtr->indicatorSpace
+	    + 2*butPtr->inset), (int) (height + 2*butPtr->inset));
+    Tk_SetInternalBorder(butPtr->tkwin, butPtr->inset);
+}
+
+
+
+#ifdef notdef
 /*
  *----------------------------------------------------------------------
  *
@@ -1323,14 +1810,13 @@ DisplayButton(clientData)
     Blt_Tile tile;
     GC tileGC;
 
-
     butPtr->flags &= ~REDRAW_PENDING;
     if ((butPtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
 	return;
     }
 
-    tile = NULL;
-    tileGC = None;
+    tile = butPtr->tile;
+    tileGC = butPtr->tileGC;
     border = butPtr->normalBorder;
     if ((butPtr->state == tkDisabledUid) && (butPtr->disabledFg != NULL)) {
 	gc = butPtr->disabledGC;
@@ -1342,8 +1828,6 @@ DisplayButton(clientData)
 	tileGC = butPtr->activeTileGC;
     } else {
 	gc = butPtr->normalTextGC;
-	tile = butPtr->tile;
-	tileGC = butPtr->tileGC;
     }
     if ((butPtr->flags & SELECTED) && (butPtr->state != tkActiveUid)
 	    && (butPtr->selectBorder != NULL) && !butPtr->indicatorOn) {
@@ -1372,7 +1856,6 @@ DisplayButton(clientData)
 
     pixmap = Tk_GetPixmap(butPtr->display, Tk_WindowId(tkwin),
 	    Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
-
     if (tileGC != NULL) {
 	Blt_SetTileOrigin(tkwin, tileGC, 0, 0);
 	XFillRectangle(butPtr->display, pixmap, tileGC, 0, 0, Tk_Width(tkwin), 
@@ -1381,6 +1864,7 @@ DisplayButton(clientData)
 	Tk_Fill3DRectangle(tkwin, pixmap, border, 0, 0, Tk_Width(tkwin),
 	    Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
     }
+
     /*
      * Display image or bitmap or text for button.
      */
@@ -1429,8 +1913,10 @@ DisplayButton(clientData)
 	    }
 	} else {
 	    XSetClipOrigin(butPtr->display, gc, x, y);
-	    XFillRectangle(butPtr->display, pixmap, gc, x, y,
-		    (unsigned) width, (unsigned) height);
+	    XSetClipMask(butPtr->display, gc, butPtr->bitmap);
+	    XCopyPlane(butPtr->display, butPtr->bitmap, pixmap, gc, 0, 0,
+		    (unsigned int) width, (unsigned int) height, x, y, 1);
+	    XSetClipMask(butPtr->display, gc, None);
 	    XSetClipOrigin(butPtr->display, gc, 0, 0);
 	}
 	y += height/2;
@@ -1471,9 +1957,16 @@ DisplayButton(clientData)
 	    x += offset;
 	    y += offset;
 	}
-	TkDisplayText(butPtr->display, pixmap, butPtr->fontPtr,
+#if (TK_MAJOR_VERSION >= 8)
+        Tk_DrawTextLayout(butPtr->display, pixmap, gc, butPtr->textLayout,
+                x, y, 0, -1);
+        Tk_UnderlineTextLayout(butPtr->display, pixmap, gc,
+                butPtr->textLayout, x, y, butPtr->underline);
+#else
+	TkDisplayText(butPtr->display, pixmap, butPtr->font,
 		butPtr->text, butPtr->numChars, x, y, butPtr->textWidth,
 		butPtr->justify, butPtr->underline, gc);
+#endif
 	y += butPtr->textHeight/2;
     }
 
@@ -1498,12 +1991,12 @@ DisplayButton(clientData)
 	    y += butPtr->borderWidth;
 	    dim -= 2*butPtr->borderWidth;
 	    if (butPtr->flags & SELECTED) {
-		GC gc;
+		GC borderGC;
 
-		gc = Tk_3DBorderGC(tkwin,(butPtr->selectBorder != NULL)
+		borderGC = Tk_3DBorderGC(tkwin,(butPtr->selectBorder != NULL)
 			? butPtr->selectBorder : butPtr->normalBorder,
 			TK_3D_FLAT_GC);
-		XFillRectangle(butPtr->display, pixmap, gc, x, y,
+		XFillRectangle(butPtr->display, pixmap, borderGC, x, y,
 			(unsigned int) dim, (unsigned int) dim);
 	    } else {
 		Tk_Fill3DRectangle(tkwin, pixmap, butPtr->normalBorder, x, y,
@@ -1524,12 +2017,12 @@ DisplayButton(clientData)
 	points[3].x = points[1].x;
 	points[3].y = points[0].y - radius;
 	if (butPtr->flags & SELECTED) {
-	    GC gc;
+	    GC borderGC;
 
-	    gc = Tk_3DBorderGC(tkwin, (butPtr->selectBorder != NULL)
+	    borderGC = Tk_3DBorderGC(tkwin, (butPtr->selectBorder != NULL)
 		    ? butPtr->selectBorder : butPtr->normalBorder,
 		    TK_3D_FLAT_GC);
-	    XFillPolygon(butPtr->display, pixmap, gc, points, 4, Convex,
+	    XFillPolygon(butPtr->display, pixmap, borderGC, points, 4, Convex,
 		    CoordModeOrigin);
 	} else {
 	    Tk_Fill3DPolygon(tkwin, pixmap, butPtr->normalBorder, points,
@@ -1578,14 +2071,14 @@ DisplayButton(clientData)
 		butPtr->borderWidth, relief);
     }
     if (butPtr->highlightWidth != 0) {
-	GC gc;
+	GC highlightGC;
 
 	if (butPtr->flags & GOT_FOCUS) {
-	    gc = Tk_GCForColor(butPtr->highlightColorPtr, pixmap);
+	    highlightGC = Tk_GCForColor(butPtr->highlightColorPtr, pixmap);
 	} else {
-	    gc = Tk_GCForColor(butPtr->highlightBgColorPtr, pixmap);
+	    highlightGC = Tk_GCForColor(butPtr->highlightBgColorPtr, pixmap);
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, butPtr->highlightWidth, pixmap);
+	Tk_DrawFocusHighlight(tkwin, highlightGC, butPtr->highlightWidth, pixmap);
     }
 
     /*
@@ -1598,6 +2091,7 @@ DisplayButton(clientData)
 	    (unsigned) Tk_Height(tkwin), 0, 0);
     Tk_FreePixmap(butPtr->display, pixmap);
 }
+#endif
 
 /*
  *--------------------------------------------------------------
@@ -1642,9 +2136,9 @@ ButtonEventProc(clientData, eventPtr)
 		    Tcl_GetCommandName(butPtr->interp, butPtr->widgetCmd));
 	}
 	if (butPtr->flags & REDRAW_PENDING) {
-	    Tk_CancelIdleCall(DisplayButton, (ClientData) butPtr);
+	    Tcl_CancelIdleCall(DisplayButton, (ClientData) butPtr);
 	}
-	Tk_EventuallyFree((ClientData) butPtr, DestroyButton);
+	Tk_EventuallyFree((ClientData) butPtr, (Tcl_FreeProc *)DestroyButton);
     } else if (eventPtr->type == FocusIn) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    butPtr->flags |= GOT_FOCUS;
@@ -1664,7 +2158,7 @@ ButtonEventProc(clientData, eventPtr)
 
     redraw:
     if ((butPtr->tkwin != NULL) && !(butPtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	butPtr->flags |= REDRAW_PENDING;
     }
 }
@@ -1710,6 +2204,7 @@ ButtonCmdDeletedProc(clientData)
     }
 }
 
+#ifdef notdef
 /*
  *----------------------------------------------------------------------
  *
@@ -1760,28 +2255,57 @@ ComputeButtonGeometry(butPtr)
 	Tk_SizeOfBitmap(butPtr->display, butPtr->bitmap, &width, &height);
 	goto imageOrBitmap;
     } else {
+#if (TK_MAJOR_VERSION >= 8)
+	int avgWidth;
+	Tk_FontMetrics fm;
+
+        Tk_FreeTextLayout(butPtr->textLayout);
+        butPtr->textLayout = Tk_ComputeTextLayout(butPtr->font,
+                butPtr->text, -1, butPtr->wrapLength, butPtr->justify, 0,
+                &butPtr->textWidth, &butPtr->textHeight);
+
+        width = butPtr->textWidth;
+        height = butPtr->textHeight;
+        avgWidth = Tk_TextWidth(butPtr->font, "0", 1);
+        Tk_GetFontMetrics(butPtr->font, &fm);
+
+        if (butPtr->width > 0) {
+            width = butPtr->width * avgWidth;
+        }
+        if (butPtr->height > 0) {
+            height = butPtr->height * fm.linespace;
+        }
+        if ((butPtr->type >= TYPE_CHECK_BUTTON) && butPtr->indicatorOn) {
+            butPtr->indicatorDiameter = fm.linespace;
+            if (butPtr->type == TYPE_CHECK_BUTTON) {
+                butPtr->indicatorDiameter = (80*butPtr->indicatorDiameter)/100;
+            }
+            butPtr->indicatorSpace = butPtr->indicatorDiameter + avgWidth;
+        }
+#else
 	butPtr->numChars = strlen(butPtr->text);
-	TkComputeTextGeometry(butPtr->fontPtr, butPtr->text,
+	TkComputeTextGeometry(butPtr->font, butPtr->text,
 		butPtr->numChars, butPtr->wrapLength, &butPtr->textWidth,
 		&butPtr->textHeight);
 	width = butPtr->textWidth;
 	height = butPtr->textHeight;
 	if (butPtr->width > 0) {
-	    width = butPtr->width * XTextWidth(butPtr->fontPtr, "0", 1);
+	    width = butPtr->width * XTextWidth(butPtr->font, "0", 1);
 	}
 	if (butPtr->height > 0) {
-	    height = butPtr->height * (butPtr->fontPtr->ascent
-		    + butPtr->fontPtr->descent);
+	    height = butPtr->height * (butPtr->font->ascent
+		    + butPtr->font->descent);
 	}
 	if ((butPtr->type >= TYPE_CHECK_BUTTON) && butPtr->indicatorOn) {
-	    butPtr->indicatorDiameter = butPtr->fontPtr->ascent
-		    + butPtr->fontPtr->descent;
+	    butPtr->indicatorDiameter = butPtr->font->ascent
+		    + butPtr->font->descent;
 	    if (butPtr->type == TYPE_CHECK_BUTTON) {
 		butPtr->indicatorDiameter = (80*butPtr->indicatorDiameter)/100;
 	    }
 	    butPtr->indicatorSpace = butPtr->indicatorDiameter
-		+ XTextWidth(butPtr->fontPtr, "0", 1);
+		+ XTextWidth(butPtr->font, "0", 1);
 	}
+#endif
     }
 
     /*
@@ -1803,6 +2327,7 @@ ComputeButtonGeometry(butPtr)
 	    + 2*butPtr->inset), (int) (height + 2*butPtr->inset));
     Tk_SetInternalBorder(butPtr->tkwin, butPtr->inset);
 }
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -1921,7 +2446,7 @@ ButtonVarProc(clientData, interp, name1, name2, flags)
     redisplay:
     if ((butPtr->tkwin != NULL) && Tk_IsMapped(butPtr->tkwin)
 	    && !(butPtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	butPtr->flags |= REDRAW_PENDING;
     }
     return (char *) NULL;
@@ -1986,7 +2511,7 @@ ButtonTextVarProc(clientData, interp, name1, name2, flags)
 
     if ((butPtr->tkwin != NULL) && Tk_IsMapped(butPtr->tkwin)
 	    && !(butPtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	butPtr->flags |= REDRAW_PENDING;
     }
     return (char *) NULL;
@@ -2010,6 +2535,7 @@ ButtonTextVarProc(clientData, interp, name1, name2, flags)
  *----------------------------------------------------------------------
  */
 
+/*ARGSUSED*/
 static void
 ButtonImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
     ClientData clientData;		/* Pointer to widget record. */
@@ -2024,7 +2550,7 @@ ButtonImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
     if (butPtr->tkwin != NULL) {
 	ComputeButtonGeometry(butPtr);
 	if (Tk_IsMapped(butPtr->tkwin) && !(butPtr->flags & REDRAW_PENDING)) {
-	    Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	    Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	    butPtr->flags |= REDRAW_PENDING;
 	}
     }
@@ -2048,6 +2574,7 @@ ButtonImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
  *----------------------------------------------------------------------
  */
 
+/*ARGSUSED*/
 static void
 ButtonSelectImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
     ClientData clientData;		/* Pointer to widget record. */
@@ -2066,25 +2593,32 @@ ButtonSelectImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
     if ((butPtr->flags & SELECTED) && (butPtr->tkwin != NULL)
 	    && Tk_IsMapped(butPtr->tkwin)
 	    && !(butPtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayButton, (ClientData) butPtr);
+	Tcl_DoWhenIdle(DisplayButton, (ClientData) butPtr);
 	butPtr->flags |= REDRAW_PENDING;
     }
 }
 
-
 int
 Blt_ButtonInit(interp)
     Tcl_Interp *interp;
 {
     static Blt_CmdSpec cmdSpecs[4] = {
-	{"tilebutton", "0.9", ButtonCmd, },
-	{"tilecheckbutton", "0.9", CheckbuttonCmd, },
-	{"tileradiobutton", "0.9", RadiobuttonCmd, },
-	{"tilelabel", "0.9", LabelCmd, },
+#if HAVE_NAMESPACES
+	{"button", ButtonCmd, },
+	{"checkbutton", CheckbuttonCmd, },
+	{"radiobutton", RadiobuttonCmd, },
+	{"label", LabelCmd, },
+#else
+	{"tilebutton", ButtonCmd, },
+	{"tilecheckbutton", CheckbuttonCmd, },
+	{"tileradiobutton", RadiobuttonCmd, },
+	{"tilelabel", LabelCmd, },
+#endif /* HAVE_NAMESPACES */
     };
     tkNormalUid = Tk_GetUid("normal");
     tkDisabledUid = Tk_GetUid("disabled");
     tkActiveUid = Tk_GetUid("active");
-    return (Blt_InitCmds(interp, cmdSpecs, 4));
+    return (Blt_InitCmds(interp, "blt::tile", cmdSpecs, 4));
 }
-#endif /* TK_MAJOR_VERSION == 4 */
+
+
