@@ -11,13 +11,12 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkFrame.c 1.68 96/02/15 18:53:30
  */
 
-static char sccsid[] = "@(#) tkFrame.c 1.59 95/06/04 14:08:47";
-
 #include "bltInt.h"
 
-#if (TK_MAJOR_VERSION == 4)
 /*
  * Defaults for frames:
  */
@@ -96,7 +95,7 @@ typedef struct {
 				 * don't request any size. */
     int height;			/* Height to request for window.  <= 0 means
 				 * don't request any size. */
-    Cursor cursor;		/* Current cursor for window, or None. */
+    Tk_Cursor cursor;		/* Current cursor for window, or None. */
     char *takeFocus;		/* Value of -takefocus option;  not used in
 				 * the C code, but used by keyboard traversal
 				 * scripts.  Malloc'ed, but may be NULL. */
@@ -112,14 +111,12 @@ typedef struct {
  * REDRAW_PENDING:		Non-zero means a DoWhenIdle handler
  *				has already been queued to redraw
  *				this window.
- * CLEAR_NEEDED;		Need to clear the window when redrawing.
  * GOT_FOCUS:			Non-zero means this widget currently
  *				has the input focus.
  */
 
 #define REDRAW_PENDING		1
-#define CLEAR_NEEDED		2
-#define GOT_FOCUS		4
+#define GOT_FOCUS		2
 
 /*
  * The following flag bits are used so that there can be separate
@@ -189,7 +186,7 @@ static Tk_ConfigSpec configSpecs[] = {
 static int		ConfigureFrame _ANSI_ARGS_((Tcl_Interp *interp,
 			    Frame *framePtr, int argc, char **argv,
 			    int flags));
-static void		DestroyFrame _ANSI_ARGS_((ClientData clientData));
+static void		DestroyFrame _ANSI_ARGS_((DestroyData *memPtr));
 static void		DisplayFrame _ANSI_ARGS_((ClientData clientData));
 static void		FrameCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
@@ -199,24 +196,31 @@ static int		FrameWidgetCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		MapFrame _ANSI_ARGS_((ClientData clientData));
 
-extern char *		TkInitFrame _ANSI_ARGS_((Tcl_Interp *interp, 
-			    Tk_Window tkwin, int toplevel, int argc, 
-			    char **argv));
+#ifdef TILE_MAINWINDOW
+extern
+#else 
+static 
+#endif
+int			TkCreateFrame _ANSI_ARGS_((ClientData clientData,
+			   Tcl_Interp *interp, int argc, char **argv,
+			   int toplevel, char *appName));
+
 
 /*
  *--------------------------------------------------------------
  *
- * FrameCmd --
+ * FrameCmd, ToplevelCmd --
  *
- *	This procedure is invoked to process the "frame" and
+ *	These procedures are invoked to process the "frame" and
  *	"toplevel" Tcl commands.  See the user documentation for
- *	details on what it does.
+ *	details on what they do.
  *
  * Results:
  *	A standard Tcl result.
  *
  * Side effects:
- *	See the user documentation.
+ *	See the user documentation.  These procedures are just wrappers;
+ *	they call ButtonCreate to do all of the real work.
  *
  *--------------------------------------------------------------
  */
@@ -228,15 +232,64 @@ FrameCmd(clientData, interp, argc, argv)
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
     char **argv;		/* Argument strings. */
+{
+    return TkCreateFrame(clientData, interp, argc, argv, 0, (char *) NULL);
+}
+
+static int
+ToplevelCmd(clientData, interp, argc, argv)
+    ClientData clientData;	/* Main window associated with
+				 * interpreter. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    char **argv;		/* Argument strings. */
+{
+    return TkCreateFrame(clientData, interp, argc, argv, 1, (char *) NULL);
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkFrameCreate --
+ *
+ *	This procedure is invoked to process the "frame" and "toplevel"
+ *	Tcl commands;  it is also invoked directly by Tk_Init to create
+ *	a new main window.  See the user documentation for the "frame"
+ *	and "toplevel" commands for details on what it does.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the user documentation.
+ *
+ *--------------------------------------------------------------
+ */
+
+#ifndef TILE_MAINWINDOW
+static 
+#endif /* TILE_MAINWINDOW */
+int
+TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
+    ClientData clientData;	/* Main window associated with interpreter.
+				 * If we're called by Tk_Init to create a
+				 * new application, then this is NULL. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    char **argv;		/* Argument strings. */
+    int toplevel;		/* Non-zero means create a toplevel window,
+				 * zero means create a frame. */
+    char *appName;		/* Should only be non-NULL if clientData is
+				 * NULL:  gives the base name to use for the
+				 * new application. */
 {
     Tk_Window tkwin = (Tk_Window) clientData;
     Frame *framePtr;
     Tk_Window new = NULL;
     char *className, *screenName, *visualName, *colormapName, *arg;
-    int i, c, length, toplevel, depth;
+    int i, c, length, depth;
     Colormap colormap;
     Visual *visual;
-    Display *display;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -246,13 +299,13 @@ FrameCmd(clientData, interp, argc, argv)
 
     /*
      * Pre-process the argument list.  Scan through it to find any
-     * "-class", "-screen", "-visual", and "-newcmap" options.  These
+     * "-class", "-screen", "-visual", and "-colormap" options.  These
      * arguments need to be processed specially, before the window
      * is configured using the usual Tk mechanisms.
      */
 
-    toplevel = (argv[0][0] == 't');
     className = colormapName = screenName = visualName = NULL;
+    colormap = None;
     for (i = 2; i < argc; i += 2) {
 	arg = argv[i];
 	length = strlen(arg);
@@ -285,7 +338,18 @@ FrameCmd(clientData, interp, argc, argv)
     if (screenName == NULL) {
 	screenName = (toplevel) ? "" : NULL;
     }
-    new = Tk_CreateWindowFromPath(interp, tkwin, argv[1], screenName);
+    if (tkwin != NULL) {
+	new = Tk_CreateWindowFromPath(interp, tkwin, argv[1], screenName);
+    } else {
+	/*
+	 * We were called from Tk_Init;  create a new application.
+	 */
+
+	if (appName == NULL) {
+	    panic("TkCreateFrame didn't get application name");
+	}
+	new = TkCreateMainWindow(interp, screenName, appName);
+    }
     if (new == NULL) {
 	goto error;
     }
@@ -302,7 +366,6 @@ FrameCmd(clientData, interp, argc, argv)
     if (colormapName == NULL) {
 	colormapName = Tk_GetOption(new, "colormap", "Colormap");
     }
-    colormap = None;
     if (visualName != NULL) {
 	visual = Tk_GetVisual(interp, new, visualName, &depth,
 		(colormapName == NULL) ? &colormap : (Colormap *) NULL);
@@ -319,73 +382,28 @@ FrameCmd(clientData, interp, argc, argv)
 	Tk_SetWindowColormap(new, colormap);
     }
 
+    /*
+     * For top-level windows, provide an initial geometry request of
+     * 200x200,  just so the window looks nicer on the screen if it
+     * doesn't request a size for itself.
+     */
+
+    if (toplevel) {
+	Tk_GeometryRequest(new, 200, 200);
+    }
+
     /*
      * Create the widget record, process configuration options, and
      * create event handlers.  Then fill in a few additional fields
      * in the widget record from the special options.
      */
 
-    display = Tk_Display(new);
-    framePtr = (Frame *) TkInitFrame(interp, new, toplevel, argc-2, argv+2);
-    if (framePtr == NULL) {
-	if (colormap != None) {
-	    Tk_FreeColormap(display, colormap);
-	}
-	return TCL_ERROR;
-    }
-    framePtr->colormap = colormap;
-    return TCL_OK;
-
-    error:
-    if (new != NULL) {
-	Tk_DestroyWindow(new);
-    }
-    return TCL_ERROR;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TkInitFrame --
- *
- *	This procedure initializes a frame or toplevel widget.  It's
- *	separate from Tk_FrameCmd so that it can be used for the
- *	main window, which has already been created elsewhere.
- *
- * Results:
- *	Returns NULL if an error occurred while initializing the
- *	frame.  Otherwise returns a pointer to the frame's widget
- *	record (for use by Tk_FrameCmd, if it was the caller).
- *
- * Side effects:
- *	A widget record gets allocated, handlers get set up, etc..
- *
- *----------------------------------------------------------------------
- */
-
-static char *
-TkInitFrame(interp, tkwin, toplevel, argc, argv)
-    Tcl_Interp *interp;			/* Interpreter associated with the
-					 * application. */
-    Tk_Window tkwin;			/* Window to use for frame or
-					 * top-level.   Caller must already
-					 * have set window's class. */
-    int toplevel;			/* Non-zero means that this is a
-					 * top-level window, 0 means it's a
-					 * frame. */
-    int argc;				/* Number of configuration arguments
-					 * (not including class command and
-					 * window name). */
-    char *argv[];			/* Configuration arguments. */
-{
-    register Frame *framePtr;
-
     framePtr = (Frame *) ckalloc(sizeof(Frame));
-    framePtr->tkwin = tkwin;
-    framePtr->display = Tk_Display(tkwin);
+    framePtr->tkwin = new;
+    framePtr->display = Tk_Display(new);
     framePtr->interp = interp;
     framePtr->widgetCmd = Tcl_CreateCommand(interp,
-	    Tk_PathName(framePtr->tkwin), FrameWidgetCmd,
+	    Tk_PathName(new), FrameWidgetCmd,
 	    (ClientData) framePtr, FrameCmdDeletedProc);
 #ifdef ITCL_NAMESPACES
     Itk_SetWidgetCommand(tkwin, framePtr->widgetCmd);
@@ -395,7 +413,7 @@ TkInitFrame(interp, tkwin, toplevel, argc, argv)
     framePtr->screenName = NULL;
     framePtr->visualName = NULL;
     framePtr->colormapName = NULL;
-    framePtr->colormap = None;
+    framePtr->colormap = colormap;
     framePtr->border = NULL;
     framePtr->borderWidth = 0;
     framePtr->relief = TK_RELIEF_FLAT;
@@ -410,18 +428,22 @@ TkInitFrame(interp, tkwin, toplevel, argc, argv)
     framePtr->tile = NULL;
     framePtr->tileGC = NULL;
 
-    Tk_CreateEventHandler(framePtr->tkwin,
-	    ExposureMask|StructureNotifyMask|FocusChangeMask,
+    Tk_CreateEventHandler(new, ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    FrameEventProc, (ClientData) framePtr);
-    if (ConfigureFrame(interp, framePtr, argc, argv, 0) != TCL_OK) {
-	Tk_DestroyWindow(framePtr->tkwin);
-	return NULL;
+    if (ConfigureFrame(interp, framePtr, argc-2, argv+2, 0) != TCL_OK) {
+	goto error;
     }
     if (toplevel) {
-	Tk_DoWhenIdle(MapFrame, (ClientData) framePtr);
+	Tcl_DoWhenIdle(MapFrame, (ClientData) framePtr);
     }
-    interp->result = Tk_PathName(framePtr->tkwin);
-    return (char *) framePtr;
+    interp->result = Tk_PathName(new);
+    return TCL_OK;
+
+    error:
+    if (new != NULL) {
+	Tk_DestroyWindow(new);
+    }
+    return TCL_ERROR;
 }
 
 /*
@@ -459,7 +481,7 @@ FrameWidgetCmd(clientData, interp, argc, argv)
 		argv[0], " option ?arg arg ...?\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    Tk_Preserve((ClientData) framePtr);
+    Tcl_Preserve((ClientData) framePtr);
     c = argv[1][0];
     length = strlen(argv[1]);
     if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
@@ -512,12 +534,12 @@ FrameWidgetCmd(clientData, interp, argc, argv)
 	}
     } else {
 	Tcl_AppendResult(interp, "bad option \"", argv[1],
-		"\":  must be cget or configure", (char *) NULL);
+		"\": must be cget or configure", (char *) NULL);
 	result = TCL_ERROR;
     }
 
     done:
-    Tk_Release((ClientData) framePtr);
+    Tcl_Release((ClientData) framePtr);
     return result;
 }
 
@@ -526,7 +548,7 @@ FrameWidgetCmd(clientData, interp, argc, argv)
  *
  * DestroyFrame --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a frame at a safe time
  *	(when no-one is using it anymore).
  *
@@ -540,10 +562,10 @@ FrameWidgetCmd(clientData, interp, argc, argv)
  */
 
 static void
-DestroyFrame(clientData)
-    ClientData clientData;	/* Info about frame widget. */
+DestroyFrame(memPtr)
+    DestroyData *memPtr;		/* Info about frame widget. */
 {
-    register Frame *framePtr = (Frame *) clientData;
+    register Frame *framePtr = (Frame *) memPtr;
 
     Tk_FreeOptions(configSpecs, (char *) framePtr, framePtr->display,
 	    framePtr->mask);
@@ -568,10 +590,9 @@ DestroyFrame(clientData)
  */
 /* ARGSUSED */
 static void
-TileChangedProc(clientData, tile, gcPtr)
+TileChangedProc(clientData, tile)
     ClientData clientData;
     Blt_Tile tile;
-    GC *gcPtr;			/* Not used */
 {
     Frame *framePtr = (Frame *)clientData;
 
@@ -579,8 +600,8 @@ TileChangedProc(clientData, tile, gcPtr)
 	GC newGC;
 	Pixmap pixmap;
 
-	pixmap = Blt_PixmapOfTile(tile);
 	newGC = NULL;
+	pixmap = Blt_PixmapOfTile(tile);
 	if (pixmap != None) {
 	    unsigned long gcMask;
 	    XGCValues gcValues;
@@ -589,14 +610,14 @@ TileChangedProc(clientData, tile, gcPtr)
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
 	    newGC = Tk_GetGC(framePtr->tkwin, gcMask, &gcValues);
-	}
+	} 
 	if (framePtr->tileGC != NULL) {
 	    Tk_FreeGC(framePtr->display, framePtr->tileGC);
 	}
 	framePtr->tileGC = newGC;
 	if (!(framePtr->flags & REDRAW_PENDING)) {
 	    Tk_DoWhenIdle(DisplayFrame, (ClientData)framePtr);
-	    framePtr->flags |= REDRAW_PENDING | CLEAR_NEEDED;
+	    framePtr->flags |= REDRAW_PENDING;
 	}
     }
 }
@@ -635,32 +656,30 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
 	    argc, argv, (char *) framePtr, flags | framePtr->mask) != TCL_OK) {
 	return TCL_ERROR;
     }
-
     if (framePtr->tile != NULL) {
 	GC newGC;
-	XGCValues gcValues;
-	unsigned long gcMask;
 	Pixmap pixmap;
 	
-	gcMask = 0;
+	newGC = NULL;
 	pixmap = Blt_PixmapOfTile(framePtr->tile);
 	if (pixmap != None) {
+	    XGCValues gcValues;
+	    unsigned long gcMask;
+
 	    gcMask = (GCTile | GCFillStyle);
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
-	} else if (framePtr->border != NULL) {
-	    gcMask = GCBackground;
-	    gcValues.background = Tk_3DBorderColor(framePtr->border)->pixel;
-	}
-	newGC = Tk_GetGC(framePtr->tkwin, gcMask, &gcValues);
+	    newGC = Tk_GetGC(framePtr->tkwin, gcMask, &gcValues);
+	} 
 	if (framePtr->tileGC != NULL) {
 	    Tk_FreeGC(framePtr->display, framePtr->tileGC);
 	}
 	framePtr->tileGC = newGC;
 	Blt_SetTileChangedProc(framePtr->tile, TileChangedProc,
-	    (ClientData)framePtr, &(framePtr->tileGC));
+	    (ClientData)framePtr);
 
-    } else if (framePtr->border != NULL) {
+    } 
+    if (framePtr->border != NULL) {
 	Tk_SetBackgroundFromBorder(framePtr->tkwin, framePtr->border);
     }
     if (framePtr->highlightWidth < 0) {
@@ -675,9 +694,9 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
 
     if (Tk_IsMapped(framePtr->tkwin)) {
 	if (!(framePtr->flags & REDRAW_PENDING)) {
-	    Tk_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
+	    Tcl_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
 	}
-	framePtr->flags |= REDRAW_PENDING|CLEAR_NEEDED;
+	framePtr->flags |= REDRAW_PENDING;
     }
     return TCL_OK;
 }
@@ -706,34 +725,33 @@ DisplayFrame(clientData)
     register Frame *framePtr = (Frame *) clientData;
     register Tk_Window tkwin = framePtr->tkwin;
     GC gc;
-    Drawable drawable;
 
     framePtr->flags &= ~REDRAW_PENDING;
-    if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
+    if ((tkwin == NULL) || !Tk_IsMapped(tkwin)) {
 	return;
     }
 
-    drawable = Tk_WindowId(tkwin);
     if (framePtr->tileGC == NULL) {
-	XClearWindow(framePtr->display, Tk_WindowId(tkwin));
-    } else {
-	drawable = Tk_GetPixmap(framePtr->display, Tk_WindowId(tkwin), 
-		      Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
-	Blt_SetTileOrigin(tkwin, framePtr->tileGC, 0, 0);
-	XFillRectangle(framePtr->display, drawable, framePtr->tileGC,
-		0, 0, Tk_Width(tkwin), Tk_Height(tkwin));
-    }
-
-    if ((framePtr->border != NULL)
-	    && (framePtr->relief != TK_RELIEF_FLAT)) {
-	Tk_Draw3DRectangle(tkwin, drawable,
+	Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
 		framePtr->highlightWidth,
-		Tk_Width(tkwin) - 2*framePtr->highlightWidth,
-		Tk_Height(tkwin) - 2*framePtr->highlightWidth,
+		Tk_Width(tkwin) - 2 * framePtr->highlightWidth,
+		Tk_Height(tkwin) - 2 * framePtr->highlightWidth,
 		framePtr->borderWidth, framePtr->relief);
+    } else {
+	Blt_SetTileOrigin(tkwin, framePtr->tileGC, 0, 0);
+	XFillRectangle(framePtr->display, Tk_WindowId(tkwin), framePtr->tileGC,
+		0, 0, Tk_Width(tkwin), Tk_Height(tkwin));
+	if ((framePtr->border != NULL) && (framePtr->relief != TK_RELIEF_FLAT)) {
+	    Tk_Draw3DRectangle(tkwin, Tk_WindowId(tkwin),
+			       framePtr->border, framePtr->highlightWidth,
+			       framePtr->highlightWidth,
+			       Tk_Width(tkwin) - 2*framePtr->highlightWidth,
+			       Tk_Height(tkwin) - 2*framePtr->highlightWidth,
+			       framePtr->borderWidth, framePtr->relief);
+	}
     }
-    if (framePtr->highlightWidth != 0) {
+     if (framePtr->highlightWidth != 0) {
 	if (framePtr->flags & GOT_FOCUS) {
 	    gc = Tk_GCForColor(framePtr->highlightColorPtr,
 		    Tk_WindowId(tkwin));
@@ -742,12 +760,7 @@ DisplayFrame(clientData)
 		    Tk_WindowId(tkwin));
 	}
 	Tk_DrawFocusHighlight(tkwin, gc, framePtr->highlightWidth,
-	      drawable);
-    }
-    if (drawable != Tk_WindowId(tkwin)) {
-	XCopyArea(framePtr->display, drawable, Tk_WindowId(tkwin), 
-	    framePtr->tileGC, 0, 0, Tk_Width(tkwin), Tk_Height(tkwin), 0, 0);
-	Tk_FreePixmap(framePtr->display, drawable);
+		Tk_WindowId(tkwin));
     }
 }
 
@@ -777,15 +790,8 @@ FrameEventProc(clientData, eventPtr)
 {
     register Frame *framePtr = (Frame *) clientData;
 
-    if ((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0)) {
-	goto redraw;
-    } else if (eventPtr->type == ConfigureNotify) {
-	/*
-	 * If the window changed size then the borders need to be
-	 * redrawn.
-	 */
-
-	framePtr->flags |= CLEAR_NEEDED;
+    if (((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0))
+	    || (eventPtr->type == ConfigureNotify)) {
 	goto redraw;
     } else if (eventPtr->type == DestroyNotify) {
 	if (framePtr->tkwin != NULL) {
@@ -797,10 +803,10 @@ FrameEventProc(clientData, eventPtr)
 		    Tcl_GetCommandName(framePtr->interp, framePtr->widgetCmd));
 	}
 	if (framePtr->flags & REDRAW_PENDING) {
-	    Tk_CancelIdleCall(DisplayFrame, (ClientData) framePtr);
+	    Tcl_CancelIdleCall(DisplayFrame, (ClientData) framePtr);
 	}
-	Tk_CancelIdleCall(MapFrame, (ClientData) framePtr);
-	Tk_EventuallyFree((ClientData) framePtr, DestroyFrame);
+	Tcl_CancelIdleCall(MapFrame, (ClientData) framePtr);
+	Tcl_EventuallyFree((ClientData) framePtr, (Tcl_FreeProc *)DestroyFrame);
     } else if (eventPtr->type == FocusIn) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    framePtr->flags |= GOT_FOCUS;
@@ -820,8 +826,8 @@ FrameEventProc(clientData, eventPtr)
 
     redraw:
     if ((framePtr->tkwin != NULL) && !(framePtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
-	framePtr->flags |= REDRAW_PENDING;
+	Tcl_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
+	framePtr->flags |= REDRAW_PENDING ;
     }
 }
 
@@ -896,9 +902,9 @@ MapFrame(clientData)
      * window manager doesn't get a false idea of its desired geometry.
      */
 
-    Tk_Preserve((ClientData) framePtr);
+    Tcl_Preserve((ClientData) framePtr);
     while (1) {
-	if (Tk_DoOneEvent(TK_IDLE_EVENTS) == 0) {
+	if (Tcl_DoOneEvent(TCL_IDLE_EVENTS) == 0) {
 	    break;
 	}
 
@@ -908,12 +914,12 @@ MapFrame(clientData)
 	 */
 
 	if (framePtr->tkwin == NULL) {
-	    Tk_Release((ClientData) framePtr);
+	    Tcl_Release((ClientData) framePtr);
 	    return;
 	}
     }
     Tk_MapWindow(framePtr->tkwin);
-    Tk_Release((ClientData) framePtr);
+    Tcl_Release((ClientData) framePtr);
 }
 
 int
@@ -922,10 +928,13 @@ Blt_FrameInit(interp)
 {
     static Blt_CmdSpec cmdSpecs[2] = 
     {
-	{ "tileframe", "0.9", FrameCmd, },
-	{ "tiletoplevel", "0.9", FrameCmd, },
+#if HAVE_NAMESPACES
+	{ "frame", FrameCmd, },
+	{ "toplevel", ToplevelCmd, },
+#else
+	{ "tileframe", FrameCmd, },
+	{ "tiletoplevel", ToplevelCmd, },
+#endif /* HAVE_NAMESPACES */
     };
-
-    return (Blt_InitCmds(interp, cmdSpecs, 2));
+    return (Blt_InitCmds(interp, "blt::tile", cmdSpecs, 2));
 }
-#endif /* TK_MAJOR_VERSION == 4 */
