@@ -11,13 +11,12 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkScrollbar.c 1.79 96/02/15 18:52:40
  */
 
-static char sccsid[] = "@(#) tkScrollbar.c 1.74 95/06/29 13:59:50";
-
 #include "bltInt.h" 
 
-#if (TK_MAJOR_VERSION == 4) 
 extern Tk_CustomOption bltTileOption;
 
 #define NORMAL_BG	"#d9d9d9" 
@@ -54,6 +53,7 @@ extern Tk_CustomOption bltTileOption;
 #define DEF_SCROLLBAR_TROUGH_MONO	WHITE
 #define DEF_SCROLLBAR_WIDTH		"15"
 
+
 /*
  * A data structure of the following type is kept for each scrollbar
  * widget managed by this file:
@@ -163,13 +163,10 @@ typedef struct {
      * Miscellaneous information:
      */
 
-    Cursor cursor;		/* Current cursor for window, or None. */
+    Tk_Cursor cursor;		/* Current cursor for window, or None. */
     char *takeFocus;		/* Value of -takefocus option;  not used in
 				 * the C code, but used by keyboard traversal
 				 * scripts.  Malloc'ed, but may be NULL. */
-    Tk_TimerToken autoRepeat;	/* Token for auto-repeat that's
-				 * currently in progress.  NULL means no
-				 * auto-repeat in progress. */
     int flags;			/* Various flags;  see below for
 				 * definitions. */
     Blt_Tile tile, activeTile;
@@ -296,7 +293,7 @@ static void		ComputeScrollbarGeometry _ANSI_ARGS_((
 static int		ConfigureScrollbar _ANSI_ARGS_((Tcl_Interp *interp,
 			    Scrollbar *scrollPtr, int argc, char **argv,
 			    int flags));
-static void		DestroyScrollbar _ANSI_ARGS_((ClientData clientData));
+static void		DestroyScrollbar _ANSI_ARGS_((DestroyData *memPtr));
 static void		DisplayScrollbar _ANSI_ARGS_((ClientData clientData));
 static void		EventuallyRedraw _ANSI_ARGS_((Scrollbar *scrollPtr));
 static void		ScrollbarCmdDeletedProc _ANSI_ARGS_((
@@ -339,7 +336,7 @@ ScrollbarCmd(clientData, interp, argc, argv)
     Tk_Window new;
 
     if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args:  should be \"",
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
 		argv[0], " pathName ?options?\"", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -397,7 +394,6 @@ ScrollbarCmd(clientData, interp, argc, argv)
     scrollPtr->lastFraction = 0.0;
     scrollPtr->cursor = None;
     scrollPtr->takeFocus = NULL;
-    scrollPtr->autoRepeat = NULL;
     scrollPtr->flags = 0;
     scrollPtr->tile = scrollPtr->activeTile = NULL;
     scrollPtr->tileGC = scrollPtr->activeTileGC = None;
@@ -454,7 +450,7 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
 		argv[0], " option ?arg arg ...?\"", (char *) NULL);
 	return TCL_ERROR;
     }
-    Tk_Preserve((ClientData) scrollPtr);
+    Tcl_Preserve((ClientData) scrollPtr);
     c = argv[1][0];
     length = strlen(argv[1]);
     if ((c == 'a') && (strncmp(argv[1], "activate", length) == 0)) {
@@ -506,7 +502,7 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
 		    TK_CONFIG_ARGV_ONLY);
 	}
     } else if ((c == 'd') && (strncmp(argv[1], "delta", length) == 0)) {
-	int xDelta, yDelta, pixels, length;
+	int xDelta, yDelta, pixels, barWidth;
 	double fraction;
 
 	if (argc != 4) {
@@ -520,21 +516,21 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
 	}
 	if (scrollPtr->vertical) {
 	    pixels = yDelta;
-	    length = Tk_Height(scrollPtr->tkwin) - 1
+	    barWidth = Tk_Height(scrollPtr->tkwin) - 1
 		    - 2*(scrollPtr->arrowLength + scrollPtr->inset);
 	} else {
 	    pixels = xDelta;
-	    length = Tk_Width(scrollPtr->tkwin) - 1
+	    barWidth = Tk_Width(scrollPtr->tkwin) - 1
 		    - 2*(scrollPtr->arrowLength + scrollPtr->inset);
 	}
-	if (length == 0) {
+	if (barWidth == 0) {
 	    fraction = 0.0;
 	} else {
-	    fraction = ((double) pixels / (double) length);
+	    fraction = ((double) pixels / (double) barWidth);
 	}
 	sprintf(interp->result, "%g", fraction);
     } else if ((c == 'f') && (strncmp(argv[1], "fraction", length) == 0)) {
-	int x, y, pos, length;
+	int x, y, pos, barWidth;
 	double fraction;
 
 	if (argc != 4) {
@@ -548,17 +544,17 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
 	}
 	if (scrollPtr->vertical) {
 	    pos = y - (scrollPtr->arrowLength + scrollPtr->inset);
-	    length = Tk_Height(scrollPtr->tkwin) - 1
+	    barWidth = Tk_Height(scrollPtr->tkwin) - 1
 		    - 2*(scrollPtr->arrowLength + scrollPtr->inset);
 	} else {
 	    pos = x - (scrollPtr->arrowLength + scrollPtr->inset);
-	    length = Tk_Width(scrollPtr->tkwin) - 1
+	    barWidth = Tk_Width(scrollPtr->tkwin) - 1
 		    - 2*(scrollPtr->arrowLength + scrollPtr->inset);
 	}
-	if (length == 0) {
+	if (barWidth == 0) {
 	    fraction = 0.0;
 	} else {
-	    fraction = ((double) pos / (double) length);
+	    fraction = ((double) pos / (double) barWidth);
 	}
 	if (fraction < 0) {
 	    fraction = 0;
@@ -685,11 +681,11 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
 	goto error;
     }
     done:
-    Tk_Release((ClientData) scrollPtr);
+    Tcl_Release((ClientData) scrollPtr);
     return result;
 
     error:
-    Tk_Release((ClientData) scrollPtr);
+    Tcl_Release((ClientData) scrollPtr);
     return TCL_ERROR;
 }
 
@@ -698,7 +694,7 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
  *
  * DestroyScrollbar --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a scrollbar at a safe time
  *	(when no-one is using it anymore).
  *
@@ -712,10 +708,10 @@ ScrollbarWidgetCmd(clientData, interp, argc, argv)
  */
 
 static void
-DestroyScrollbar(clientData)
-    ClientData clientData;	/* Info about scrollbar widget. */
+DestroyScrollbar(memPtr)
+    DestroyData *memPtr;	/* Info about scrollbar widget. */
 {
-    register Scrollbar *scrollPtr = (Scrollbar *) clientData;
+    register Scrollbar *scrollPtr = (Scrollbar *) memPtr;
 
     /*
      * Free up all the stuff that requires special handling, then
@@ -758,16 +754,19 @@ DestroyScrollbar(clientData)
  *----------------------------------------------------------------------
  */
 static void
-TileChangedProc(clientData, tile, gcPtr)
+TileChangedProc(clientData, tile)
     ClientData clientData;
     Blt_Tile tile;
-    GC *gcPtr;
 {
     Scrollbar *scrollPtr = (Scrollbar *)clientData;
 
     if (scrollPtr->tkwin != NULL) {
 	GC newGC;
 	Pixmap pixmap;
+	GC *gcPtr;
+
+	gcPtr = (tile == scrollPtr->tile) 
+	    ? &(scrollPtr->tileGC) : &(scrollPtr->activeTileGC);
 
 	newGC = NULL;
 	pixmap = Blt_PixmapOfTile(tile);
@@ -852,11 +851,13 @@ ConfigureScrollbar(interp, scrollPtr, argc, argv, flags)
     }
     if (scrollPtr->activeTile != NULL) {
 	Pixmap pixmap;
+	GC gc;
 
 	Blt_SetTileChangedProc(scrollPtr->activeTile, TileChangedProc,
-	    (ClientData)scrollPtr, &(scrollPtr->activeTileGC));
+	    (ClientData)scrollPtr);
 
 	pixmap = Blt_PixmapOfTile(scrollPtr->activeTile);
+	gc = NULL;
 	if (pixmap != None) {
 	    unsigned int gcMask;
 
@@ -864,31 +865,33 @@ ConfigureScrollbar(interp, scrollPtr, argc, argv, flags)
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
 
-	    new = Tk_GetGC(scrollPtr->tkwin, gcMask, &gcValues);
+	    gc = Tk_GetGC(scrollPtr->tkwin, gcMask, &gcValues);
 	}
 	if (scrollPtr->activeTileGC != NULL) {
 	    Tk_FreeGC(scrollPtr->display, scrollPtr->activeTileGC);
 	}
-	scrollPtr->activeTileGC = new;
+	scrollPtr->activeTileGC = gc;
     }
     if (scrollPtr->tile != NULL) {
 	Pixmap pixmap;
+	GC gc;
 
 	Blt_SetTileChangedProc(scrollPtr->tile, TileChangedProc,
-	    (ClientData)scrollPtr, &(scrollPtr->tileGC));
+	    (ClientData)scrollPtr);
 	pixmap = Blt_PixmapOfTile(scrollPtr->tile);
+	gc = NULL;
 	if (pixmap != None) {
 	    unsigned int gcMask;
 
 	    gcMask = (GCTile | GCFillStyle);
 	    gcValues.fill_style = FillTiled;
 	    gcValues.tile = pixmap;
-	    new = Tk_GetGC(scrollPtr->tkwin, gcMask, &gcValues);
+	    gc = Tk_GetGC(scrollPtr->tkwin, gcMask, &gcValues);
 	}
 	if (scrollPtr->tileGC != NULL) {
 	    Tk_FreeGC(scrollPtr->display, scrollPtr->tileGC);
 	}
-	scrollPtr->tileGC = new;
+	scrollPtr->tileGC = gc;
     } 
 
     Tk_SetBackgroundFromBorder(scrollPtr->tkwin, scrollPtr->bgBorder);
@@ -986,7 +989,6 @@ DisplayScrollbar(clientData)
 	    Tk_Width(tkwin) - 2*scrollPtr->highlightWidth,
 	    Tk_Height(tkwin) - 2*scrollPtr->highlightWidth,
 	    scrollPtr->borderWidth, scrollPtr->relief);
-
     if (scrollPtr->tile != NULL) {
 	Blt_SetTileOrigin(tkwin, scrollPtr->tileGC, 0, 0);
 	fillGC = scrollPtr->tileGC;
@@ -1049,7 +1051,6 @@ DisplayScrollbar(clientData)
     /*
      * Display the bottom or right arrow.
      */
-
     fillGC = NULL;
     if (scrollPtr->activeField == BOTTOM_ARROW) {
 	border = scrollPtr->activeBorder;
@@ -1083,7 +1084,6 @@ DisplayScrollbar(clientData)
 	points[2].x = Tk_Width(tkwin) - scrollPtr->inset;
 	points[2].y = width/2 + scrollPtr->inset;
     }
-
     if (fillGC != NULL) {
 	XFillPolygon(scrollPtr->display, pixmap, fillGC, points, 3, Convex, 
 		CoordModeOrigin);
@@ -1198,9 +1198,10 @@ ScrollbarEventProc(clientData, eventPtr)
 		    scrollPtr->widgetCmd));
 	}
 	if (scrollPtr->flags & REDRAW_PENDING) {
-	    Tk_CancelIdleCall(DisplayScrollbar, (ClientData) scrollPtr);
+	    Tcl_CancelIdleCall(DisplayScrollbar, (ClientData) scrollPtr);
 	}
-	Tk_EventuallyFree((ClientData) scrollPtr, DestroyScrollbar);
+	Tcl_EventuallyFree((ClientData) scrollPtr, 
+			   (Tcl_FreeProc *)DestroyScrollbar);
     } else if (eventPtr->type == ConfigureNotify) {
 	ComputeScrollbarGeometry(scrollPtr);
 	EventuallyRedraw(scrollPtr);
@@ -1434,7 +1435,7 @@ EventuallyRedraw(scrollPtr)
 	return;
     }
     if ((scrollPtr->flags & REDRAW_PENDING) == 0) {
-	Tk_DoWhenIdle(DisplayScrollbar, (ClientData) scrollPtr);
+	Tcl_DoWhenIdle(DisplayScrollbar, (ClientData) scrollPtr);
 	scrollPtr->flags |= REDRAW_PENDING;
     }
 }
@@ -1443,8 +1444,12 @@ int
 Blt_ScrollbarInit(interp)
     Tcl_Interp *interp;
 {
-    static Blt_CmdSpec cmdSpec = { "tilescrollbar", "0.9", ScrollbarCmd, };
-
-    return (Blt_InitCmd(interp, &cmdSpec));
+    static Blt_CmdSpec cmdSpec = { 
+#if HAVE_NAMESPACES
+	"scrollbar", ScrollbarCmd, 
+#else 
+	"tilescrollbar", ScrollbarCmd, 
+#endif /* HAVE_NAMESPACES */
+    };
+    return (Blt_InitCmd(interp, "blt::tile", &cmdSpec));
 }
-#endif /* TK_MAJOR_VERSION == 4 */
